from ltk.actions.action import *

class ReferenceAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def reference_add_action(self, filename, doc_id):
        if self._check_filename(filename, doc_id):
            material = []
            while True:
                while True:
                    prompt_message = "Reference Material file: "
                    # Python 2
                    # file_input = raw_input(prompt_message)
                    # End Python 2
                    # Python 3
                    file_input = input(prompt_message)
                    if not file_input:
                        logger.warning("You must enter a path to reference material")
                        continue
                    ref_file = os.path.abspath(os.path.expanduser(file_input))
                    if os.path.isfile(ref_file):
                        break
                    else:
                        logger.error(ref_file+" could not be found")
                prompt_message = "Reference Material Name: "
                # Python 2
                # name_input = raw_input(prompt_message)
                # End Python 2
                # Python 3
                name_input = input(prompt_message)
                prompt_message = "Reference Material Description: "
                # Python 2
                # desc_input = raw_input(prompt_message)
                # End Python 2
                # Python 3
                desc_input = input(prompt_message)
                reference = {'file': ref_file}
                if name_input:
                    reference['name'] = name_input
                else:
                    reference['name'] = os.path.basename(ref_file)
                if desc_input:
                    reference['description'] = desc_input
                material.append(reference)
                if not yes_no_prompt("Would you like to add another reference material?", default_yes=False):
                    break
            if doc_id:
                document_id = filename
            else:
                doc_entity = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(filename))
                if not doc_entity:
                    logger.error("{0} could not be found in local database".format(self.norm_path(filename)))
                    return
                document_id = doc_entity['id']
            for reference in material:
                response = self.api.document_add_reference(document_id, reference)
                if response.status_code == 404:
                    logger.warning("The reference material could not be added because the document could not be found in Lingotek.  The document may still be in the process of uploading.")
                elif response.status_code != 202:
                    logger.info("The reference material could not be added")
                    logger.error(response.json()['messages'])
                else:
                    logger.info("{0} ({1}) has been added to the document".format(reference['name'], response.json()['properties']['id']))

    
    def reference_list_action(self, filename, doc_id):
        if self._check_filename(filename, doc_id):
            if doc_id:
                document_id = filename
            else:
                doc_entity = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(filename))
                if not doc_entity:
                    logger.error("{0} could not be found in local database".format(self.norm_path(filename)))
                    return
                document_id = doc_entity['id']
            self._list_reference_material(document_id)

    def reference_download_action(self, filename, doc_id, get_all, path):
        if not path:
            path = self.path
        if self._check_filename(filename, doc_id):
            if doc_id:
                document_id = filename
            else:
                doc_entity = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(filename))
                if not doc_entity:
                    logger.error("{0} could not be found in local database".format(self.norm_path(filename)))
                    return
                document_id = doc_entity['id']
            table = self._list_reference_material(document_id)
            tablemap = {}
            for row in table:
                tablemap.update({row[0]: {'name': row[1], 'id': row[2]}})
            if len(tablemap) > 0:
                chosen_list = []
                if get_all:
                    chosen_list = tablemap.values()
                while not len(chosen_list) > 0:
                    prompt_message = 'Reference materials to download: (Separate indices by comma) '
                    # Python 2
                    # choice = raw_input(prompt_message)
                    # End Python 2
                    # Python 3
                    choice = input(prompt_message)
                    # End Python 3
                    try:
                        choices = (choice.replace(", ",",")).split(",")
                        for index in choices:
                            chosen_list.append(tablemap[int(index)])
                    except ValueError:
                        logger.error('Some unexpected, non-integer value was included')
                        chosen_list = []
                    except KeyError:
                        logger.error('An index not in the list was included')
                        chosen_list = []
                for reference in chosen_list:
                    response = self.api.document_download_reference(document_id, reference['id'])
                    if response.status_code == 404:
                        logger.error("{0} ({1}) not found".format(reference['name'], reference['id']))
                    elif response.status_code == 200:
                        self._download_reference(response, path, reference['name'])
                    else:
                        logger.info("{0} ({1}) could not be downloaded".format(reference['name'], reference['id']))
                        logger.error(response.json()['messages'])

    def reference_remove_action(self, filename, doc_id, remove_all):
       if self._check_filename(filename, doc_id):
           if doc_id:
               document_id = filename
           else:
               doc_entity = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(filename))
               if not doc_entity:
                   logger.error("{0} could not be found in local database".format(self.norm_path(filename)))
                   return
               document_id = doc_entity['id']
           table = self._list_reference_material(document_id)
           tablemap = {}
           for row in table:
               tablemap.update({row[0]: {'name': row[1], 'id': row[2]}})
           if len(tablemap) > 0:
               chosen_list = []
               if remove_all:
                   chosen_list = tablemap.values()
               while not len(chosen_list) > 0:
                   prompt_message = 'Reference materials to remove: (Separate indices by comma) '
                   # Python 2
                   # choice = raw_input(prompt_message)
                   # End Python 2
                   # Python 3
                   choice = input(prompt_message)
                   # End Python 3
                   try:
                       choices = (choice.replace(", ",",")).split(",")
                       for index in choices:
                           chosen_list.append(tablemap[int(index)])
                   except ValueError:
                       logger.error('Some unexpected, non-integer value was included')
                       chosen_list = []
                   except KeyError:
                       logger.error('An index not in the list was included')
                       chosen_list = []
               for reference in chosen_list:
                   response = self.api.document_remove_reference(document_id, reference['id'])
                   if response.status_code == 404:
                       logger.error("{0} ({1}) not found".format(reference['name'], reference['id']))
                   elif response.status_code == 204:
                       logger.info("{0} ({1}) deleted".format(reference['name'], reference['id']))
                   else:
                       logger.info("{0} ({1}) could not be deleted".format(reference['name'], reference['id']))
                       logger.error(response.json()['messages'])

    def _check_filename(self, filename, doc_id):
        if doc_id:
            #if document ID is specified, no need to validate the filename.  Just send the ID and let the API handle the error if the ID doesn't exist
            return True
        if os.path.isfile(filename):
            foundfile = self.get_doc_filenames_in_path(filename)
            if len(foundfile) == 0:
                logger.warning(filename+" has not been added yet.")
                return False
            elif len(foundfile) == 1:
                return True
            else:
                logger.warning("Only manage reference material on one file at a time")
                return False
        elif os.path.isdir(filename):
            logger.error(filename+" is not a file")
            return False
        else:
            logger.error(filename+" could not be found")
            return False

    def _list_reference_material(self, document_id):
        response = self.api.document_list_reference(document_id)
        if response.status_code == 404:
            logger.warning("The document could not be found in Lingotek.")
            return []
        elif response.status_code != 200:
            logger.info("The reference material list could not be retrieved")
            logger.error(response.json()['messages'])
            return []
        else:
            if response.json()['properties']['size'] > 0:
                materials = response.json()['entities']
                index = 0
                table = []
                for material in materials:
                    row = [index, material['properties']['name'], material['properties']['id']]
                    if 'description' in material['properties'] and material['properties']['description']:
                        row.append(material['properties']['description'])
                    table.append(row)
                    index += 1
                print(tabulate(table, headers=['','Name','ID','Description']))
                return table
            else:
                print("There is no reference material attached to this document")
                return []
    
    def _download_reference(self, response, path, name):
        filepath = os.path.join(path, name)
        if os.path.isfile(filepath):
            if not yes_no_prompt("There is already a file {0}.  Would you like to overwrite it?".format(filepath), default_yes=False):
                return
        try:
            with open(filepath, 'wb') as file:
                for chunk in response.iter_content(1024):
                    file.write(chunk)
        except IOError as e:
            print(e.errno)
            print(e)
            return
        logger.info("Downloaded {0}".format(filepath))


import socket

def check_for_connection():
    try:
        host = socket.gethostbyname("www.google.com")
        s = socket.create_connection((host, 80), 2)
        return True
    except:
        pass
    return False



from ltk.actions.action import *
from tabulate import tabulate

class ListAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def list_action(self, **kwargs):
        if 'id_type' in kwargs and kwargs['id_type']:
            id_type = kwargs['id_type']
            if id_type == 'workflow':
                self.list_workflows()
            elif id_type == 'locale':
                self.list_locales()
            elif id_type == 'format':
                self.list_formats()
            elif id_type == 'filter':
                self.list_filters()
            elif id_type == 'remote':
                self.list_remote()

        elif 'hide_docs' in kwargs and 'title' in kwargs and 'show_dests' in kwargs:
            self.list_ids(kwargs['hide_docs'], kwargs['title'], kwargs['show_dests'])

        else: logger.error("Error on command line input for 'ltk list'")

    def list_filters(self):
        response = self.api.list_filters()
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get filters')
        filter_entities = response.json()['entities']
        # print ('Filters: id, created, title')
        table = []
        headers=["ID","Created","Title"]
        for entry in sorted(filter_entities, key=lambda entry: entry['properties']['upload_date'], reverse=True):
            properties = entry['properties']
            title = properties['title']
            filter_id = properties['id']
            upload_date = time.strftime("%Y-%m-%d", time.localtime(int(properties['upload_date']/1000)))
            is_public = " (public)" if properties['is_public'] else ""
            table.append([
                str(filter_id), #ID
                str(upload_date), #Created
                title #Title
            ])
        print(tabulate(table, headers=headers))

    def list_formats(self):
        format_info = self.api.get_document_formats()
        format_mapper = detect_format(None, True)

        format_list = {}
        for format_name in sorted(set(format_info.values())):
            format_list[format_name] = []

        for extension in format_mapper.keys():
            key = format_mapper[extension]
            if key not in format_list:
                format_list[key] = []
            format_list[key].append(extension)

        print("Lingotek Cloud accepts content using any of the formats listed below. File formats will be auto-detected for the extensions as specified below. Alternatively, formats may be specified explicitly upon add. Lingotek supports variations and customizations on these formats with filters.\n")
        table = []
        for k,v in sorted(format_list.items()):
            table.append([
                k, # Format
                ' | '.join([str(x) for x in v]) # Auto-detected File Extensions
            ])
        print(tabulate(table, headers=["Format","Auto-detected File Extensions"]))
        
    def list_ids(self, hide_docs, title=False, downloads=False):
        try:
            """ lists ids of list_type specified """
            folders = self.folder_manager.get_file_names()
            if len(folders):
                # underline("Folder path")
                table = []
                for folder in folders:
                    if title:
                        table.append([folder])
                    else:
                        table.append([self.get_relative_path(folder)])
                print(tabulate(table, headers=["Folder Path"]))
                if hide_docs:
                    return
                print("")
            elif hide_docs:
                print("No added folders")
                return
            ids = []
            titles = []
            locales = []
            dests = []
            max_length = 0
            entries = self.doc_manager.get_all_entries()
            for entry in entries:
                # if entry['file_name'].startswith(cwd.replace(self.path, '')):
                ids.append(entry['id'])
                try:
                    if title:
                        name = entry['name']
                    else:
                        name = self.get_relative_path(self.norm_path(entry['file_name']))
                    if len(name) > max_length:
                        max_length = len(name)
                    titles.append(name)
                except (IndexError, KeyError) as e:
                    log_error(self.error_file_name, e)
                    titles.append("        ")
                try:
                    locales.append(entry['locales'])
                except KeyError:
                    locales.append([])
                if downloads:
                    try:
                        folder = entry['download_folder']
                        dests.append(folder)
                    except KeyError:
                        dests.append('')
            if not ids:
                print ('No local documents')
                return
            if max_length > 90:
                max_length = 90
            table = []
            for i in range(len(ids)):
                filetitle = titles[i]
                if len(filetitle) > max_length:
                    filetitle = filetitle[(len(titles[i])-30):]
                for locale in locales[i]:
                    locale.replace('_', '-')
                if title:
                    headers = ["Title","Lingotek ID","Locales"]
                else:
                    headers = ["Filename","Lingotek ID","Locales"]
                if downloads:
                    headers.append("Target Download Folder")
                    table.append([
                        filetitle,
                        str(ids[i]),
                        ', '.join([str(x).replace('_', '-') for x in locales[i]]),
                        str(dests[i])
                    ])
                else:
                    table.append([
                        filetitle,
                        str(ids[i]),
                        ', '.join([str(x).replace('_', '-') for x in locales[i]])
                    ])
            print(tabulate(table, headers=headers))
        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on list: "+str(e))

    def list_locales(self):
        locale_info = []
        table = []
        response = self.api.list_locales()
        if response.status_code != 200:
            raise exceptions.RequestFailedError("Failed to get locale codes")
        locale_json = response.json()
        for entry in locale_json:
            locale_code = locale_json[entry]['locale'].replace('_','-')
            language = locale_json[entry]['language_name']
            country = locale_json[entry]['country_name']
            locale_info.append((locale_code, language, country))
        for locale in sorted(locale_info):
            if not len(locale[2]):  # Arabic
                table.append(["{0}".format(locale[0]), "({0})".format(locale[1])])
            else:
                table.append(["{0}".format(locale[0]), "({0}, {1})".format(locale[1], locale[2])])
        print(tabulate(table))
    def list_remote(self):
        """ lists ids of all remote documents """
        response = self.api.list_documents(self.project_id)
        if response.status_code == 204:
            print("No documents to report")
            return
        elif response.status_code != 200:
            if check_response(response):
                raise_error(response.json(), "Failed to get status of documents", True)
            else:
                raise_error("", "Failed to get status of documents", True)
        else:
            table = []
            headers=["Name", "Lingotek ID"]
            for entry in response.json()['entities']:
                title = entry['properties']['title']
                id = entry['properties']['id']
                table.append([title, id])
            print(tabulate(table, headers=headers))

    def list_workflows(self):
        try:
            response = self.api.list_workflows(self.community_id)
            if response.status_code != 200:
                raise_error(response.json(), "Failed to list workflows")

            table = []
            headers=["Workflow Name", "ID"]
            for entry in response.json()['entities']:
                title = entry['properties']['title']
                id = entry['properties']['id']
                table.append([title, id])
            print(tabulate(table, headers=headers))

        except:
            logger.error("An error occurred while attempting to connect to remote.")



from ltk.actions.action import *
from ltk.git_auto import Git_Auto

DEFAULT_COMMIT_MESSAGE  = "Translations updated for "

class PullAction(Action):
    def __init__(self, path, download):
        Action.__init__(self, path)
        self.git_auto = Git_Auto(path)
        self.download = download

    def pull_translations(self, locale_code, locale_ext, no_ext, auto_format):
        if auto_format == False and self.auto_format_option == 'on':
            auto_format = True
        first_in_message = True
        documents_added = True
        git_commit_message = DEFAULT_COMMIT_MESSAGE
        try:
            if 'clone' in self.download_option and not locale_ext or (no_ext and not locale_ext):
                locale_ext = False
            else:
                locale_ext = True
            entries = self.doc_manager.get_all_entries()
            if not locale_code:
                if entries:
                    for entry in entries:
                        git_commit_message += ("" if first_in_message else "; ") + entry['name'] + ": "
                        first_in_message = False
                        first_locale = True
                        try:
                            locales = entry['locales']
                            for locale in locales:
                                locale = locale.replace('_','-')
                                git_commit_message += ("" if first_locale else ", ") + locale
                                first_locale = False
                                self.download.download_action(entry['id'], locale, auto_format, locale_ext=locale_ext)
                        except KeyError:
                            self.download.download_action(entry['id'], None, auto_format, locale_ext=locale_ext)
                            git_commit_message = git_commit_message[:-1]
                else:
                    logger.info("No documents have been added")
                    documents_added = False
            else:
                document_ids = self.doc_manager.get_doc_ids()
                if document_ids:
                    for document_id in document_ids:
                        git_commit_message += ("" if first_in_message else "; ") + self.doc_manager.get_doc_by_prop("id", document_id)["name"] + ": " + locale_code
                        first_in_message = False
                        first_locale = True
                        self.download.download_action(document_id, locale_code, auto_format, locale_ext=locale_ext)
                else:
                    logger.info("No documents have been added")
                    documents_added = False

            ###
            ### Adding, committing, and pushing to Github has been moved to DownloadAction ###
            ###
            # if documents_added:
            #     config_file_name, conf_parser = self.init_config_file()
            #     git_autocommit = conf_parser.get('main', 'git_autocommit')
            #     if git_autocommit in ['True', 'on']:    #'True' option for back-support of previous layout
            #         if not self.git_auto.repo_is_defined:
            #             if(self.git_auto.repo_exists()):
            #                 self.git_auto.commit(git_commit_message)
            #                 self.git_auto.push()

        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on pull: "+str(e))



__all__ = [ "action",
            "add_action",
            "clean_action",
            "clone_action",
            "config_action",
            "download_action",
            "filters_action",
            "import_action",
            "init_action",
            "list_action",
            "move_action",
            "pull_action",
            "push_action",
            "request_action",
            "rm_action",
            "status_action",
            "reference_action"
          ]



from ltk.actions.action import *
from tabulate import tabulate

class ConfigAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)
        self.config_file_name, self.conf_parser = self.init_config_file()
        self.print_config = True

    def config_action(self, **kwargs):
        try:
            if 'locale' in kwargs and kwargs['locale']:
                self.set_locale(kwargs['locale'])

            if 'workflow_id' in kwargs and kwargs['workflow_id']:
                self.set_workflow_id(kwargs['workflow_id'])

            if 'download_folder' in kwargs and kwargs['download_folder']:
                self.set_download_folder(kwargs['download_folder'])

            if 'latest_document' in kwargs and kwargs['latest_document']:
                self.set_always_check_latest_doc(kwargs['latest_document'])

            if 'clone_option' in kwargs and kwargs['clone_option']:
                self.set_clone_option(kwargs['clone_option'])

            if 'finalized_file' in kwargs and kwargs['finalized_file']:
                self.set_finalized_file_option(kwargs['finalized_file'])

            if 'unzip_file' in kwargs and kwargs['unzip_file']:
                self.set_unzip_file_option(kwargs['unzip_file'])

            if 'target_locales' in kwargs and kwargs['target_locales']:
                self.set_target_locales(kwargs['target_locales'])

            if 'locale_folder' in kwargs and kwargs['locale_folder']:
                self.set_locale_folder(kwargs['locale_folder'])

            if 'remove_locales' in kwargs and kwargs['remove_locales']:
                self.remove_locales(kwargs['remove_locales'])

            self.update_config_parser_info()

            if 'git' in kwargs and kwargs['git']:
                self.set_git_autocommit(kwargs['git'])

            if 'git_credentials' in kwargs and kwargs['git_credentials']:
                self.set_git_credentials()

            if 'append_option' in kwargs and kwargs['append_option']:
                self.set_append_option(kwargs['append_option'])

            if 'auto_format' in kwargs and kwargs['auto_format']:
                self.set_auto_format_option(kwargs['auto_format'])
            
            if 'metadata_prompt' in kwargs and kwargs['metadata_prompt']:
                self.set_metadata_prompt(kwargs['metadata_prompt'])

            if 'metadata_fields' in kwargs and kwargs['metadata_fields']:
                self.set_metadata_fields(kwargs['metadata_fields'])

            if 'metadata_defaults' in kwargs and kwargs['metadata_defaults']: #handle this last in case a prior argument caused an error
                self.set_metadata_defaults()

            self.print_output()

        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on config: "+str(e))

    def print_output(self):
        download_dir = "None"
        if self.download_dir and str(self.download_dir) != 'null':
            download_dir = self.download_dir
        locale_folders_str = "None"
        if self.locale_folders:
            locale_folders_str = json.dumps(self.locale_folders).replace("{","").replace("}","").replace("_","-")
        current_git_username = self.conf_parser.get('main', 'git_username')
        current_git_password = self.conf_parser.get('main', 'git_password')
        git_output = ('active' if self.git_autocommit in ['True', 'on'] else 'inactive')
        if self.git_autocommit in ['True', 'on']:
            if current_git_username != "":
                git_output += (' (' + current_git_username + ', password:' + ('YES' if current_git_password != '' else 'NO')) + ')'
            else:
                git_output += (' (password:YES)' if current_git_password != '' else ' (no credentials set, recommend SSH key)')
        if self.print_config:
            watch_locales = set()
            for locale in self.watch_locales:
                watch_locales.add(locale.replace('_','-'))
            watch_locales = ','.join(target for target in watch_locales)
            if str(watch_locales) == "[]" or not watch_locales:
                watch_locales = "None"
            """ print ('Host: {0}\nLingotek Project: {1} ({2})\nLocal Project Path: {3}\nCommunity ID: {4}\nWorkflow ID: {5}\n'
                'Default Source Locale: {6}\nAlways Check Latest Document: {7}\nClone Option: {8}\nDownload Finalized Files: {9}\nAuto Format: {10}\nDownload Folder: {11}\nTarget Locales: {12}\nTarget Locale Folders: {13}\nGit Auto-commit: {14}\nAppend Option: {15}'.format(
                self.host, self.project_id, self.project_name, self.path, self.community_id, self.workflow_id, self.locale, self.clone_option, self.finalized_file, self.auto_format_option,
                download_dir, watch_locales, locale_folders_str, git_output, self.append_option)) """
            table = [
                ["Host", self.host], 
                ["Lingotek Project", '{0} ({1})'.format(self.project_id, self.project_name)],
                ["Local Project Path", self.path],
                ["Community ID", self.community_id],
                ["Workflow ID", self.workflow_id],
                ["Default Source Locale", self.locale],
                ["Always Check Latest Document", self.always_check_latest_doc],
                ["Clone Option", self.clone_option],
                ["Download Finalized Files", self.finalized_file],
                ["Auto Format", self.auto_format_option],
                ["Default Download Folder", download_dir],
                ["Target Locales", watch_locales],
                ["Target Locale Folders", locale_folders_str],
                ["Git Auto-commit", git_output],
                ["Append Option", self.append_option.title()],
                ["Metadata Wizard Fields", ",\n".join(self.metadata_fields)],
                ["Always Prompt for Metadata", "on" if self.metadata_prompt else "off"],
                ["Default Metadata", "\n".join("{}:{}".format(key, value) for key, value in self.default_metadata.items())]
            ]
            if self.finalized_file == 'on':
                table.append(["Unzip Finalized File", self.unzip_file])
            print("Configuration Options")
            print(tabulate(table))
        self.print_config = True

    def remove_locales(self, clear_locales):
        log_info = "Removed all locale specific download folders."
        self.locale_folders = {}
        locale_folders_str = json.dumps(self.locale_folders)
        self.update_config_file('locale_folders', locale_folders_str, self.conf_parser, self.config_file_name, log_info)

    def set_append_option(self, append_option):
        self.print_config = True
        self.append_option = append_option
        if append_option in {'none', 'full'} or append_option[:append_option.find(':')+1] in {'number:', 'name:'}:
            set_option = True
            if append_option[:3] == 'num':
                try: int(append_option[7:])
                except ValueError:
                    logger.warning('Error: Input after "number" must be an integer')
                    self.print_config = False
            elif append_option[:4] == 'name' and len(append_option) <= 5:
                logger.warning('Error: No input given after "name"')
                self.print_config = False
            if not self.conf_parser.has_option('main', 'append_option'):
                self.update_config_file('append_option', 'none', self.conf_parser, self.config_file_name, 'Update: Added optional file location appending (ltk config --help)')
            if self.print_config:
                log_info = 'Append option set to ' + append_option
                self.update_config_file('append_option', append_option, self.conf_parser, self.config_file_name, log_info)
        else:
            logger.warning('Error: Invalid value for "-a" / "--append_option": Must be one of "none", "full", "number:", or "name:"')
            self.print_config = False

    def set_auto_format_option(self, auto_format_option):
        log_info = 'Turned auto format '+auto_format_option
        if auto_format_option == 'on':
            self.auto_format_option = 'on'
            self.update_config_file('auto_format', auto_format_option, self.conf_parser, self.config_file_name, log_info)
        elif auto_format_option == 'off':
            self.auto_format_option = 'off'
            self.update_config_file('auto_format', auto_format_option, self.conf_parser, self.config_file_name, log_info)
        else:
            logger.warning('Error: Invalid value for "-f" / "--auto_format": Must be either "on" or "off"')
            print_config = False

    def set_clone_option(self, clone_option, print_info=True):
        self.clone_action = clone_option
        if print_info:
            log_info = 'Turned clone '+clone_option
        else:
            log_info = ''
        if clone_option == 'on':
            download_option = 'clone'
            self.download_option = download_option

            self.update_config_file('clone_option', clone_option, self.conf_parser, self.config_file_name, log_info)
            self.update_config_file('download_option', download_option, self.conf_parser, self.config_file_name, '')
        elif clone_option == 'off':
            if self.download_dir == '':
                new_download_option = 'same'
                self.download_option = new_download_option

                self.update_config_file('clone_option', clone_option, self.conf_parser, self.config_file_name, log_info)
                self.update_config_file('download_option', new_download_option, self.conf_parser, self.config_file_name, '')
                self.update_config_file('download_folder', self.download_dir, self.conf_parser, self.config_file_name, '')
            else:
                new_download_option = 'folder'
                self.download_option = new_download_option
                self.update_config_file('clone_option', clone_option, self.conf_parser, self.config_file_name, log_info)
                self.update_config_file('download_option', new_download_option, self.conf_parser, self.config_file_name, '')
        else:
            logger.warning('Error: Invalid value for "-c" / "--clone_option": Must be either "on" or "off"')
            print_config = False

    def set_always_check_latest_doc(self, always_check_latest_doc_option, print_info=True):
        if print_info:
            log_info = 'Turned always check latest document ' + always_check_latest_doc_option
        else:
            log_info = ''
        if always_check_latest_doc_option == 'on' or always_check_latest_doc_option == 'off':
            self.update_config_file('always_check_latest_doc', always_check_latest_doc_option, self.conf_parser, self.config_file_name, log_info)
        else:
            logger.warning('Error: Invalid value for "-ld" / "--latest_document": Must be either "on" or "off"')

    def set_finalized_file_option(self, finalized_file, print_info=True):
        if finalized_file:
            finalized_file = finalized_file.lower()
        if print_info:
            log_info = 'Turned finalized file download ' + finalized_file
        else:
            log_info = ''
        if finalized_file == 'on' or finalized_file == 'off':
            self.finalized_file = finalized_file
            self.update_config_file('finalized_file', finalized_file, self.conf_parser, self.config_file_name, log_info)
            if self.finalized_file == 'on':
                unzip_file = self.prompt_unzip_file_option()
                self.set_unzip_file_option(unzip_file)
        else:
            logger.warning('Error: Invalid value for "-ff" / "--finalized_file": Must be either "on" or "off"')
            self.print_config = False

    def prompt_unzip_file_option(self):
        unzip_file = 'on'
        try:
            confirm = 'none'
            while confirm not in ['on', 'On', 'ON', 'off', 'Off', '']:
                prompt_message = 'Would you like to turn finalized file UNZIP on or off? [ON/off]: '
                # Python 2
                # confirm = raw_input(prompt_message)
                # End Python 2
                # Python 3
                confirm = input(prompt_message)
                # End Python 3
                if confirm in ['on', 'On', 'ON', 'off', 'Off', '']:
                    if confirm in ['on', 'On', 'ON', '']:
                        unzip_file = 'on'
                    else:
                        unzip_file = 'off'

        except KeyboardInterrupt:
            # Python 2
            # logger.info("\nInit canceled")
            # End Python 2
            # Python 3
            logger.error("\nInit canceled")
            # End Python 3
            return

        return unzip_file

    def set_unzip_file_option(self, unzip_file, print_info=True):
        if unzip_file:
            unzip_file = unzip_file.lower()
        if print_info:
            log_info = 'Turned finalized file unzip ' + unzip_file
        else:
            log_info = ''
        if unzip_file == 'on' or unzip_file == 'off':
            self.unzip_file = unzip_file
            self.update_config_file('unzip_file', unzip_file, self.conf_parser, self.config_file_name, log_info)
        else:
            logger.warning('Error: Invalid value for "-u" / "--unzip_file": Must be either "on" or "off"')
            self.print_config = False

    def set_download_folder(self, download_folder):
        if download_folder == '--none':
            if self.download_dir == "":
                pass
            else:
                new_download_option = 'same'
                self.download_option = new_download_option
                self.update_config_file('download_folder',"", self.conf_parser, self.config_file_name, "")
                if self.download_option != 'clone':
                    if self.watch_locales != None and len(self.locale_folders) != 0:
                        new_download_option = 'folder'
                    else:
                        new_download_option = 'same'
                    self.download_option = new_download_option
                    log_info = 'Removed download folder'
                    self.update_config_file('download_option', new_download_option, self.conf_parser, self.config_file_name, log_info)
        else:
            download_path = self.norm_path(download_folder)
            if os.path.exists(os.path.join(self.path,download_path)):
                self.download_dir = download_path
                log_info = 'Set download folder to {0}'.format(download_path)
                self.update_config_file('download_folder', download_path, self.conf_parser, self.config_file_name, log_info)

                if self.download_option != 'clone':
                    new_download_option = 'folder'
                    self.download_option = new_download_option
                    self.update_config_file('download_option', new_download_option, self.conf_parser, self.config_file_name, "")
            else:
                logger.warning('Error: Invalid value for "-d" / "--download_folder": The folder {0} does not exist'.format(os.path.join(self.path,download_path)))
                print_config = False

    def set_git_autocommit(self, git_autocommit):
        # if self.git_autocommit == 'True' or self.git_auto.repo_exists(self.path):
        #     log_info = 'Git auto-commit status changed from {0}active'.format(
        #         ('active to in' if self.git_autocommit == "True" else 'inactive to '))
        #     config_file = open(self.config_file_name, 'w')
        #     if self.git_autocommit == "True":
        #         self.update_config_file('git_autocommit', 'False', self.conf_parser, self.config_file_name, log_info)
        #         self.git_autocommit = "False"
        #     else:
        #         self.update_config_file('git_autocommit', 'True', self.conf_parser, self.config_file_name, log_info)
        #         self.git_autocommit = "True"
        self.git_autocommit = git_autocommit
        log_info = 'Turned git auto-commit ' + git_autocommit
        if git_autocommit in ['on', 'off']:
            self.update_config_file('git_autocommit', git_autocommit, self.conf_parser, self.config_file_name, log_info)
        else:
            logger.warning('Error: Invalid value for "-g" / "--clone_option": Must be either "on" or "off"')
            print_config = False

    def set_git_credentials(self):
        if "nt" not in os.name:
            # Python 2
            # git_username = raw_input('Username: ')
            # End Python 2
            # Python 3
            git_username = input('Username: ')
            # End Python 3
            git_password = getpass.getpass()
            if git_username in ['None', 'none', 'N', 'n', '--none']:
                git_username = ""
                log_info = "Git username disabled"
            else:
                log_info = 'Git username set to ' + git_username
            self.update_config_file('git_username', git_username, self.conf_parser, self.config_file_name, log_info)
            if git_password in ['None', 'none', 'N', 'n', '--none']:
                git_password = ""
                log_info = "Git password disabled"
            else:
                log_info = 'Git password set'
            self.update_config_file('git_password', self.git_auto.encrypt(git_password), self.conf_parser, self.config_file_name, log_info)
        else:
            error("Only SSH Key access is enabled on Windows")
            git_username = ""
            git_password = ""

    def set_locale(self, locale):
        self.locale = locale
        log_info = 'Project default locale has been updated to {0}'.format(self.locale)
        self.update_config_file('default_locale', locale, self.conf_parser, self.config_file_name, log_info)

    def set_locale_folder(self, locale_folders):
        count = 0
        folders_count = len(locale_folders)
        folders_string = ""
        log_info = ""
        mult_folders = False
        for folder in locale_folders:
            count += 1
            if not folder[0] or not folder[1]:
                logger.warning("Please specify a valid locale and a directory for that locale.")
                print_config = False
                continue
            locale = folder[0].replace("-","_")
            if not is_valid_locale(self.api, locale):
                logger.warning(str(locale+' is not a valid locale. See "ltk list -l" for the list of valid locales'))
                print_config = False
                continue
            if folder[1] == '--none':
                folders_count -= 1
                if locale in self.locale_folders:
                    self.locale_folders.pop(locale, None)
                    logger.info("Removing download folder for locale "+str(locale)+"\n")
                else:
                    logger.info("The locale "+str(locale)+" already has no download folder.\n")
                    print_config = False
                continue
            path = self.norm_path(os.path.abspath(folder[1]))
            if os.path.exists(os.path.join(self.path,path)):
                taken_locale = self.is_locale_folder_taken(locale, path)
                if taken_locale:
                    logger.info("The folder "+str(path)+" is already taken by the locale "+str(taken_locale)+".\n")
                    print_config = False
                    continue
                else:
                    # print("path of new locale folder: "+path)
                    self.locale_folders[locale] = path
            else:
                logger.warning('Error: Invalid value for "-p" / "--locale_folder": Path "'+path+'" does not exist')
                print_config = False
                continue
            folders_string += str(locale) + ": " + str(path)
            if count < len(locale_folders):
                folders_string += ", "
        if len(folders_string):
            if folders_count > 1:
                log_info = 'Adding locale folders {0}'.format(folders_string)
            else:
                log_info = 'Adding locale folder for {0}'.format(folders_string)
        locale_folders_str = json.dumps(self.locale_folders)
        self.update_config_file('locale_folders', locale_folders_str, self.conf_parser, self.config_file_name, log_info)

    def set_target_locales(self, target_locales):
        locales = []
        print(target_locales)
        for locale in target_locales:
            locales.extend(locale.split(','))
        if len(locales) > 0 and (locales[0].lower() == 'none' or locales[0].lower() == '--none'):
            log_info = 'Removing all target locales'
            self.update_config_file('watch_locales', '', self.conf_parser, self.config_file_name, log_info)
        else:
            target_locales = get_valid_locales(self.api,locales,'added')
            target_locales_str = ','.join(target for target in target_locales)
            if len(target_locales_str) > 0:
                log_info = 'Set target locales to {0}'.format(target_locales_str)
                self.update_config_file('watch_locales', target_locales_str, self.conf_parser, self.config_file_name, log_info)
                self.watch_locales = target_locales

    def set_workflow_id(self, workflow_id):
        response = self.api.patch_project(self.project_id, workflow_id)
        if response.status_code != 204:
            raise_error(response.json(), 'Something went wrong trying to update workflow_id of project')
        self.workflow_id = workflow_id
        log_info = 'Project default workflow has been updated to {0}'.format(workflow_id)
        self.update_config_file('workflow_id', workflow_id, self.conf_parser, self.config_file_name, log_info)
        self.conf_parser.set('main', 'workflow_id', workflow_id)

    def update_config_parser_info(self):
        # clone_option = self.conf_parser.get('main', 'clone_option')
        # download_folder = self.conf_parser.get('main', 'download_folder')
        # download_option = self.conf_parser.get('main', 'download_option')
        # if 'download_option' == 'same' or (clone_option == "off" and download_folder == "null"):
        #     self.update_config_file('download_option', 'folder', self.conf_parser, self.config_file_name, "")
        if not self.conf_parser.has_option('main', 'git_autocommit'):
            self.update_config_file('git_autocommit', 'False', self.conf_parser, self.config_file_name, 'Update: Added \'git auto-commit\' option (ltk config --help)')
            self.update_config_file('git_username', '', self.conf_parser, self.config_file_name, 'Update: Added \'git username\' option (ltk config --help)')
            self.update_config_file('git_password', '', self.conf_parser, self.config_file_name, 'Update: Added \'git password\' option (ltk config --help)')
        self.git_autocommit = self.conf_parser.get('main', 'git_autocommit')

    def set_metadata_defaults(self):
        self.default_metadata = self.metadata_wizard(set_defaults=True)
        self.update_config_file('default_metadata', json.dumps(self.default_metadata), self.conf_parser, self.config_file_name, "Updated default metadata to {0}".format(self.default_metadata))

    def set_metadata_prompt(self, option):
        if option.lower() == 'on':
            self.metadata_prompt = True
            self.update_config_file('metadata_prompt', 'on', self.conf_parser, self.config_file_name, 'Update: Metadata prompt set to ON')
        elif option.lower() == 'off':
            self.metadata_prompt = False
            self.update_config_file('metadata_prompt', 'off', self.conf_parser, self.config_file_name, 'Update: Metadata prompt set to OFF')
        else:
            logger.warning("The flag for the metadata prompt only takes the arguments 'on' or 'off'")

    def set_metadata_fields(self, fields):
        if fields.lower() == 'all':
            self.metadata_fields = METADATA_FIELDS
        else:
            new_fields = fields.split(",")
            if len(new_fields) == 0 or fields.isspace():
                logger.error("You must set at least one field")
                return
            for field in new_fields:
                if field not in METADATA_FIELDS:
                    logger.warning("{0} is not a valid metadata field".format(field))
                    return
            self.metadata_fields = new_fields
        self.update_config_file('metadata_fields', json.dumps(self.metadata_fields), self.conf_parser, self.config_file_name, "Updated metadata wizard fields to {0}".format(self.metadata_fields))


''' Python Dependencies '''
import ctypes
import getpass
import os
import subprocess

''' Internal Dependencies '''
from ltk.actions.action import *
from ltk.actions.config_action import ConfigAction
from ltk.git_auto import Git_Auto

''' Globals '''
HIDDEN_ATTRIBUTE = 0x02

class InitAction():
    def __init__(self, path):
        self.path = path
        self.turn_clone_on = True
        #Action.__init__(self, path)

    def init_action(self, host, access_token, client_id, project_path, folder_name, workflow_id, default_locale, browser, delete, reset):
        # Tabulate is used to help create the tables on the init
        client_id = 'ab33b8b9-4c01-43bd-a209-b59f933e4fc4' if not client_id else client_id
        try:
            # check if Lingotek directory already exists
            to_init = self.reinit(host, project_path, delete, reset)
            # print("to_init: "+str(to_init))
            if not to_init:
                return
            elif to_init is not True:
                access_token = to_init
            ran_oauth = False
            if not access_token:
                access_token = self.check_global(host)
                if not access_token or reset:
                    if 'myaccount' not in host and 'clone' not in host:
                        logger.info("Connecting to " + host)
                    if browser:
                        from ltk.auth import run_oauth
                        access_token = run_oauth(host, client_id)
                        ran_oauth = True
                    else:
                        logger.info('---------------------------')
                        logger.info('CONNECT TO LINGOTEK')
                        logger.info('---------------------------')
                        self.api = ApiCalls(host, '')
                        # Python 2
                        username = raw_input('Username: ')
                        # End Python 2
                        # Python 3
#                         username = input('Lingotek Username: ')
                        # End Python 3
                        password = getpass.getpass()
                        if 'myaccount.lingotek.com' in host:
                            login_host = 'https://sso.lingotek.com'
                        elif 'clone.lingotek.com' in host:
                            login_host = 'https://clonesso.lingotek.com'
                        else:
                            if (host.find('sso') != -1):
                                host_env = host.split('sso')[0]
                                login_host = host_env + 'sso.lingotek.com'
                            else:
                                host_env = host.split('.')[0]
                                login_host = host_env + 'sso.lingotek.com'
                            print(login_host)
                        if self.api.login(login_host, username, password):
                            retrieved_token = self.api.authenticate(login_host)
                            if retrieved_token:
                                print('\nAuthentication successful\n')
                                access_token = retrieved_token
                                ran_oauth = True
                        if not ran_oauth:
                            print('Authentication failed.  Initialization canceled.')
                            return
            # print("access_token: "+str(access_token))
            if ran_oauth:
                # create or overwrite global file
                self.create_global(access_token, host)

            self.api = ApiCalls(host, access_token)
            # create a directory
            try:
                if not os.path.isdir(os.path.join(project_path, CONF_DIR)):
                    os.mkdir(os.path.join(project_path, CONF_DIR))

                # if on Windows system, set directory properties to hidden
                if os.name == 'nt':
                    try:
                        subprocess.call(["attrib", "+H", os.path.join(project_path, CONF_DIR)])
                    except KeyboardInterrupt:
                        raise
                    except Exception as e:
                        logger.error("Error on init: "+str(e))
                    # # Python 2
                    ret = ctypes.windll.kernel32.SetFileAttributesW(unicode(os.path.join(project_path, CONF_DIR)), HIDDEN_ATTRIBUTE)
                    # # End Python 2
                    # # Python 3
#                     # ret = ctypes.windll.kernel32.SetFileAttributesW(os.path.join(project_path, CONF_DIR), HIDDEN_ATTRIBUTE)
                    # # End Python 3
                    # if(ret != 1):   # return value of 1 signifies success
                    #     pass

            except OSError as e:
                #logger.info(e)
                pass
            except IOError as e:
                print(e.errno)
                print(e)

            logger.info('Initializing project...\n')
            config_file_name = os.path.join(project_path, CONF_DIR, CONF_FN)
            # create the config file and add info
            config_file = open(config_file_name, 'w')

            self.config_parser = ConfigParser()
            self.config_parser.add_section('main')
            self.config_parser.set('main', 'access_token', access_token)
            self.config_parser.set('main', 'host', host)
            # self.config_parser.set('main', 'root_path', project_path)
            # get community id
            community_info = self.api.get_communities_info()
            if not community_info:
                from ltk.auth import run_oauth
                access_token = run_oauth(host, client_id)
                self.create_global(access_token, host)
                community_info = self.api.get_communities_info()
                if not community_info:
                    raise exceptions.RequestFailedError("Unable to get user's list of communities")

            if len(community_info) == 0:
                raise exceptions.ResourceNotFound('You are not part of any communities in Lingotek Cloud')
            if len(community_info) > 1:
                logger.info('---------------------------')
                logger.info('SELECT LINGOTEK COMMUNITY')
                logger.info('---------------------------')
                community_id, community_name = self.display_choice('community', community_info)
            else:
                for id in community_info:
                    community_id = id
                #community_id = community_info.iterkeys().next()  --- iterkeys() is not in python 3
            if community_id != None:
                self.config_parser.set('main', 'community_id', community_id)
                response = self.api.list_projects(community_id)
                if response.status_code == 204:#no projects in community
                    project_info = []
                elif response.status_code == 200:
                    project_info = self.api.get_project_info(community_id)
                else:
                    try:
                        raise_error(response.json(), 'Something went wrong trying to find projects in your community')
                    except:
                        logger.error('Something went wrong trying to find projects in your community')
                        return
                project_id = None
                project_name = None
                confirm = 'none'
                logger.info('---------------------------')
                logger.info('SELECT LINGOTEK PROJECT')
                logger.info('---------------------------')
                if len(project_info) > 0:
                    project_id, project_name = self.display_choice('project', project_info)
                else:
                    logger.info('There are no projects')
                if project_id != None:
                    self.config_parser.set('main', 'project_id', project_id)
                    if project_name != None:
                        self.config_parser.set('main', 'project_name', project_name)
                if not project_id:
                    project_id, project_name = self.create_new_project(folder_name, community_id, workflow_id)
                    self.config_parser.set('main', 'project_id', project_id)
                    self.config_parser.set('main', 'project_name', project_name)

                # get workflow
                logger.info('---------------------------------------------------------------')
                logger.info('SELECT WORKFLOW TEMPLATE TO COPY AS A NEW PROJECT WORKFLOW')
                logger.info('---------------------------------------------------------------')
                workflow_id, workflow_updated = self.set_workflow(community_id, project_id)
                if(workflow_updated):
                    self.api.patch_project(project_id, workflow_id)
                self.config_parser.set('main', 'workflow_id', workflow_id)

                # print out locale codes
                logger.info('---------------------------')
                logger.info('SET SOURCE LANGUAGE')
                logger.info('---------------------------')
                self.locale_info = self.print_locale_codes()

                # get source locale
                selected_source_locale = self.set_source_locale()
                self.config_parser.set('main', 'default_locale', selected_source_locale)

                # get target locale(s)
                target_locales = self.set_target_locales()
                self.config_parser.set('main', 'watch_locales', target_locales)

                # get download location
                download_path = self.set_download_path(project_path)
                self.config_parser.set('main', 'download_folder', download_path)

                # get most recent document
                always_check_latest_doc = yes_no_prompt('Always check for the latest document?', default_yes=False)
                if always_check_latest_doc:
                    check = 'on'
                else:
                    check = 'off'
                self.config_parser.set('main', 'always_check_latest_doc', check)
                # ask about advanced settings
                if yes_no_prompt('Would you like to configure advanced options?', default_yes=False):
                    self.show_advanced_settings()

                logger.info("\nAll finished. Use ltk -h to learn more about using Lingotek Filesystem Connector.")
                self.config_parser.write(config_file)
                config_file.close()
        except KeyboardInterrupt:
            # Python 2
            logger.info("\nInit canceled")
            # End Python 2
            # Python 3
#             logger.error("\nInit canceled")
            # End Python 3
            return
        except Exception as e:
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on init: "+str(e))

    def show_advanced_settings(self):
        # git auto-commit
        username, encrypted_password = self.set_git_autocommit()
        if not username == None and not encrypted_password == None:
            self.config_parser.set('main', 'git_autocommit', 'on')
            self.config_parser.set('main', 'git_username', username)
            self.config_parser.set('main', 'git_password', encrypted_password)
        else:
            self.config_parser.set('main', 'git_autocommit', 'off')
            self.config_parser.set('main', 'git_username', '')
            self.config_parser.set('main', 'git_password', '')

        # toggle clone on/off
        print("\n--------------------------------")
        print("CLONE OPTION:")
        print("Toggle clone download option \'on\' and \'off\'.\n\nTurning clone \'on\': Translations will be downloaded to a cloned folder structure, where the root folder for each locale is the locale folder specified in config or a locale folder inside of the default download folder. If a default download folder is not set, then translations will be downloaded to the directory where the project was initialized.\n\nTurning clone \'off\': If a download folder is specified, downloaded translations will download to that folder, but not in a cloned folder structure. If no download folder is specified, downloaded translations will go to the same folder as their corresponding source files.")
        self.turn_clone_on = self.set_clone_option()
        if self.turn_clone_on:
            self.config_parser.set('main', 'clone_option', 'on')
        else:
            self.config_parser.set('main', 'clone_option', 'off')

        # toggle auto-format on/off
        print("--------------------------------")
        print("AUTO-FORMAT:")
        print("Toggle auto format option \'on\' and \'off\'. Applies formatting during download.")
        turn_auto_format_on = self.set_auto_format_option()
        if turn_auto_format_on:
            self.config_parser.set('main', 'auto_format', 'on')
        else:
            self.config_parser.set('main', 'auto_format', 'off')

        # change append options
        print("--------------------------------")
        print("APPEND OPTION:")
        print('Change the format of the default name given to documents on the Lingotek system.\nDefine file information to append to document names as none, full, number:+a number of folders down to include (e.g. number:2), or name:+a name of a directory to start after if found in file path (e.g. name:dir). Default option is none.')
        append_option = self.set_append_option()
        if not append_option == None:
            self.config_parser.set('main', 'append_option', append_option)

        # Toggle finalized file download option
        print("\n--------------------------------")
        print("DOWNLOAD FINALIZED FILE:")
        print("Toggle finalized file download option 'on' or 'off'. Turning this option on downloads the finalized file instead of the raw translation. " +
              "A finalized file is typically a file that has undergone some sort of post editing like Desktop Publishing after the translation has completed.")
        self.finalized_file = self.set_finalized_file_option()
        self.config_parser.set('main', 'finalized_file', self.finalized_file)

        if self.finalized_file == 'on':
            print("\n--------------------------------")
            print("UNZIP FINALIZED FILE:")
            print("Toggle UNZIP finalized file option 'on' or 'off'. Turning this option on unzips the file from TMS. Turning it off leaves the file unzipped.")
            self.unzip_file = self.prompt_unzip_file_option()
            self.config_parser.set('main', 'unzip_file', self.unzip_file)

        # Set default metadata
        print("--------------------------------")
        print("METADATA:")
        self.set_metadata_defaults()
        print('')
        self.set_metadata_prompt()
        print('')
        self.set_metadata_fields()

    def check_global(self, host):
        # check for a global config file and return the access token
        home_path = os.path.expanduser('~')
        sys_file = os.path.join(home_path, SYSTEM_FILE)
        if os.path.isfile(sys_file):
            # get the access token
            print("Using configuration in file "+str(sys_file))
            conf_parser = ConfigParser()
            conf_parser.read(sys_file)
            if conf_parser.has_section(host) and conf_parser.get(host, 'host') == host:
                return conf_parser.get(host, 'access_token')

        return None

    def display_choice(self, display_type, info, w_id='Project Default'):
        if display_type == 'community':
            prompt_message = 'Select Lingotek Community ID: '
        elif display_type == 'project':
            prompt_message = 'Select Project ID [create new]: '
        elif display_type == 'append option':
            prompt_message = 'Which append option should be used? '
        else:
            raise exceptions.ResourceNotFound("Cannot display info asked for")
        mapper = choice_mapper(info)
        choice = 'none-chosen'
        if display_type != 'workflow':
            while choice not in mapper:
                # Python 2
                choice = raw_input(prompt_message)
                # End Python 2
                # Python 3
#                 choice = input(prompt_message)
                # End Python 3
                try:
                    if display_type == 'project':
                        if choice == '':
                            return None, None
                        else:
                            choice = int(choice)
                    else:
                        choice = int(choice)
                except ValueError:
                    print("That's not a valid option!")
        for v in mapper[choice]:
            print(v)
            logger.info('\nSelected "{0}" {1}.\n'.format(mapper[choice][v], display_type))
            return v, mapper[choice][v]

    def reinit(self, host, project_path, delete, reset):
        if is_initialized(project_path) and not reset:
            logger.warning('This project is already initialized!')
            if not delete:
                return False
            option = yes_no_prompt("Are you sure you want to delete the current project? " + \
                    "This will also delete the project in your community.", default_yes=True)
            # confirm if deleting existing folder
            if option:
                # delete the corresponding project online
                logger.info('Deleting old project folder and creating new one...')
                config_file_name = os.path.join(project_path, CONF_DIR, CONF_FN)
                if os.path.isfile(config_file_name):
                    old_config = ConfigParser()
                    old_config.read(config_file_name)
                    project_id = old_config.get('main', 'project_id')
                    access_token = old_config.get('main', 'access_token')
                    self.api = ApiCalls(host, access_token)
                    response = self.api.delete_project(project_id)
                    if response.status_code != 204 and response.status_code != 404:
                        try:
                            error = response.json()['messages'][0]
                            raise exceptions.RequestFailedError(error)
                        except (AttributeError, IndexError):
                            raise exceptions.RequestFailedError("Failed to delete and re-initialize project")
                    # delete existing folder
                    to_remove = os.path.join(project_path, CONF_DIR)
                    shutil.rmtree(to_remove)
                else:
                    raise exceptions.ResourceNotFound("Cannot find config file, please re-initialize project")
                return access_token
            else:
                return False
        return True

    def create_global(self, access_token, host):
        """
        create a .lingotek file in user's $HOME directory
        """
        try:
            # go to the home dir
            home_path = os.path.expanduser('~')
            file_name = os.path.join(home_path, SYSTEM_FILE)
            global_parser = ConfigParser()
            if os.path.isfile(file_name):
                global_parser.read(file_name)
                # if on Windows, temporarily unhide the .lingotek file so we can write to it
                if os.name == 'nt':
                    try:
                        subprocess.call(["attrib", "-H", file_name])
                    except KeyboardInterrupt:
                        raise
                    except Exception as e:
                        logger.error("Error on init: "+str(e))
            sys_file = open(file_name, 'w')

            if global_parser.has_section(host):
                global_parser.set(host, 'access_token', access_token)
                global_parser.set(host, 'host', host)
            else:
                global_parser.add_section(host)
                global_parser.set(host, 'access_token', access_token)
                global_parser.set(host, 'host', host)

            global_parser.write(sys_file)
            sys_file.close()
        except KeyboardInterrupt:
            raise
        except Exception as e:
            logger.error("Error on init: "+str(e))
        # # if on Windows, set file properties to hidden
        if os.name == 'nt':
            try:
                subprocess.call(["attrib", "+H", file_name])
            except KeyboardInterrupt:
                raise
            except Exception as e:
                logger.error("Error on init: "+str(e))

    def create_new_project(self, folder_name, community_id, workflow_id):
        prompt_message = "Please enter a new Lingotek project name [%s]: " % folder_name
        # Python 2
        project_name = raw_input(prompt_message)
        # End Python 2
        # Python 3
#         project_name = input(prompt_message)
        # End Python 3
        if not project_name:
            project_name = folder_name
        response = self.api.add_project(project_name, community_id, workflow_id)
        if response.status_code != 201:
            try:
                raise_error(response.json(), 'Failed to add current project to Lingotek Cloud')
            except:
                logger.error('Failed to add current project to Lingotek Cloud')
                return
        project_id = response.json()['properties']['id']

        return project_id, project_name


    def set_workflow(self, community_id, project_id):
        response = self.api.list_workflows(community_id)
        workflows = response.json()['entities']
        workflow_info = {}

        for workflow in workflows:
            workflow_info[workflow['properties']['id']] = workflow['properties']['title']

        if len(workflow_info) > 0:
            confirm = 'none'
            mapper = choice_mapper(workflow_info)
            choice = 'none-chosen'
            prompt_message = 'Select workflow ID [Project Default]: '
            # Python 2
            choice = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             choice = input(prompt_message)
            # End Python 3
            try:
                if choice == '':
                    workflow_id, workflow_name = 'Project Default', 'Project Default'
                    logger.info('\nKept "{0}" {1}.\n'.format(workflow_name, 'workflow'))
                    return workflow_id, False 
                else:
                    choice = int(choice)
                    for v in mapper[choice]:
                        print(v)
                        logger.info('\nSelected "{0}" {1}.\n'.format(mapper[choice][v], 'workflow'))   
                        workflow_id, workflow_name = v, mapper[choice][v]
                        return workflow_id, True
            except ValueError:
                print('Not a valid option')

    def print_locale_codes(self):
        locale_info = []
        response = self.api.list_locales()
        if response.status_code != 200:
            raise exceptions.RequestFailedError("Failed to get locale codes")
        locale_json = response.json()
        locale_dict = {}
        for entry in locale_json:
            locale_code = locale_json[entry]['locale'].replace('_','-').replace('\'', '')
            language = locale_json[entry]['language_name']
            country = locale_json[entry]['country_name']
            locale_info.append((locale_code, language, country))
            locale_dict[locale_code] = (language, country),

        locale_info = sorted(locale_info)
        table = []
        for locale in locale_info:
            if not len(locale[2]):  # Arabic
                table.append(["{0}".format(locale[0]), "{0}".format(locale[1])])
            else:
                table.append(["{0}".format(locale[0]), "{0}, {1}".format(locale[1], locale[2])])
        print(tabulate(table, headers=["Code", "Locale Name"]))
        return locale_dict

    def set_source_locale(self):
        selected_locale = ''
        keep_prompting = True
        while selected_locale not in self.locale_info.keys():
            prompt_message = '\nWhat is the default locale for your source content? [en-US]: '
            # Python 2
            locale = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             locale = input(prompt_message)
            # End Python 3
            if(locale == ''):
                selected_locale = 'en-US'
                keep_prompting = False
            else:
                if(locale not in self.locale_info.keys()):
                    logger.warning('\'{0}\' is not a valid locale'.format(locale))
                else:
                    selected_locale = locale
                    keep_prompting = False

        logger.info("Set source locale to: {0}\n".format(selected_locale))
        return selected_locale

    def set_target_locales(self):
        user_input = 'not given'
        locales = []
        prompt_for_input = True
        while prompt_for_input:
            prompt_message = 'What default target locales would you like to translate into (e.g. fr-FR, ja-JP)? [None]: '
            # Python 2
            user_input = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             user_input = input(prompt_message)
            # End Python 3
            if(user_input == '' or user_input == 'none'):
                prompt_for_input = False
            else:
                locales = user_input.replace(" ", "").split(",")

                prompt_for_input = False
                # make sure the locales given are valid
                for l in locales:
                    if l not in self.locale_info:
                        print("Please provide valid locales as a comma seperated list (e.g. fr-FR, ja-JP)\n")
                        prompt_for_input = True

        if(user_input == 'none' or user_input == ''):
            logger.info("Set target locales to: None\n")
            return ""
        elif(len(locales) > 0):
            logger.info("Set target locales to: {0}\n".format(', '.join(locales)))
            return ','.join(locales)

    def set_download_path(self, project_path):
        download_path = ''
        keep_prompting = True
        while keep_prompting:
            prompt_message = 'Where would you like translations to be downloaded? [.]: '
            # Python 2
            path = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             path = input(prompt_message)
            # End Python 3

            if(path == '' or path == '.'):
                # set download path to the current directory
                download_path = self.norm_path(project_path, '.')
            else:
                download_path = self.norm_path(project_path, path)

            if os.path.exists(os.path.join(project_path, download_path)):
                if(path == '' or path == '.'):
                    logger.info("Set download folder to the current working directory\n")
                else:
                    logger.info("Set download folder to: {0}\n".format(download_path))

                keep_prompting = False
            else:
                logger.warning('Error: The folder {0} does not exist\n'.format(os.path.join(project_path,download_path)))

        return download_path

    def norm_path(self, project_path, file_location):
        # print("original path: "+str(file_location))
        if file_location:
            file_location = os.path.normpath(file_location)
            # abspath=os.path.abspath(file_location)
            # print("abspath: "+str(os.path.abspath(os.path.expanduser(file_location))))
            # print("self.path: "+self.path)
            # print("cwd: "+str(os.getcwd()))
            norm_path = os.path.abspath(os.path.expanduser(file_location)).replace(self.path, '')
            # print("normalized path: "+norm_path)
            # print("joined path: "+str(os.path.join(self.path,file_location)))
            # if file_location == ".." and self.path.rstrip('/') in norm_path:
            #     return norm_path.replace(self.path.rstrip('/'), '')
            if file_location is not "." and ".." not in file_location and os.path.exists(os.path.join(self.path,file_location)):
                # print("returning original path: "+str(file_location))
                return file_location.replace(self.path, '')
            elif ".." in file_location and file_location != "..":
                # print("returning norm path: "+norm_path)
                return norm_path.replace(self.path,'')
            if not os.path.exists(os.path.join(self.path,norm_path)) and os.path.exists(os.path.join(self.path,file_location)):
                # print("Starting path at project directory: "+file_location.replace(self.path, ''))
                return os.path.abspath(os.path.expanduser(file_location.replace(self.path, ''))).replace(self.path, '')
            elif file_location == "..":
                return os.path.abspath(os.path.expanduser(file_location.replace(self.path, ''))).replace(self.path, '')
            return norm_path
        else:
            return None
#This can never be reached.  Commenting it out for now
#        # go to the home dir
#        home_path = os.path.expanduser('~')
#        file_name = os.path.join(home_path, SYSTEM_FILE)
#        self.config_parser = ConfigParser()
#        if os.path.isfile(file_name):
#            self.config_parser.read(file_name)
#        sys_file = open(file_name, 'w')
#
#        if self.config_parser.has_section(host):
#            self.config_parser.set(host, 'access_token', access_token)
#            self.config_parser.set(host, 'host', host)
#        else:
#            self.config_parser.add_section(host)
#            self.config_parser.set(host, 'access_token', access_token)
#            self.config_parser.set(host, 'host', host)
#
#        self.config_parser.write(sys_file)
#        sys_file.close()

    def set_git_autocommit(self):
        if not(os.name == 'nt'):
            option = yes_no_prompt('Would you like to use Git auto-commit?', default_yes=False)
            if option:
                # get git credentials
                return self.get_git_credentials()

        return None, None

    def get_git_credentials(self):
        git_username = None
        encrypted_password = None

        if not os.name == 'nt':
            # Python 2
            git_username = raw_input('Username: ')
            # End Python 2
            # Python 3
#             git_username = input('Username: ')
            # End Python 3
            git_password = getpass.getpass()
            if git_username in ['None', 'none', 'N', 'n', '--none']:
                git_username = ""
                # log_info = "Git username disabled"
            else:
                log_info = 'Git username set to ' + git_username
            # self.update_config_file('git_username', git_username, self.conf_parser, self.config_file_name, log_info)
            if git_password in ['None', 'none', 'N', 'n', '--none']:
                git_password = ""
                # log_info = "Git password disabled"
            else:
                log_info = 'Git password set'
            # self.update_config_file('git_password', self.git_auto.encrypt(git_password), self.conf_parser, self.config_file_name, log_info)
        # else:
        #     error("Only SSH Key access is enabled on Windows")
        #     git_username = ""
        #     git_password = ""

        if not encrypted_password == None:
            git_auto = Git_Auto(self.path)
            encrypted_password = git_auto.encrypt(git_password)

        return git_username, encrypted_password

    def set_clone_option(self):
        turn_clone_on = True
        confirm = 'none'
        while confirm != 'on' and confirm != 'On' and confirm != 'ON' and confirm != 'off' and confirm != 'Off' and confirm != '':
            prompt_message = 'Would you like to turn clone on or off? [ON/off]: '
            # Python 2
            confirm = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             confirm = input(prompt_message)
            # End Python 3
            if confirm in ['on', 'On', 'ON', 'off', 'Off', '']:
                if confirm in ['on', 'On', 'ON', '']:
                    logger.info("Clone set to ON\n")
                    turn_clone_on = True
                    return turn_clone_on
                else:
                    logger.info("Clone set to OFF\n")
                    turn_clone_on = False
                    return turn_clone_on

        return turn_clone_on

    def set_auto_format_option(self):
        turn_auto_format_on = True
        confirm = 'none'
        while confirm != 'on' and confirm != 'On' and confirm != 'ON' and confirm != 'off' and confirm != 'Off' and confirm != '':
            prompt_message = 'Would you like to turn auto-format on or off? [ON/off]: '
            # Python 2
            confirm = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             confirm = input(prompt_message)
            # End Python 3
            if confirm in ['on', 'On', 'ON', 'off', 'Off', '']:
                if confirm in ['on', 'On', 'ON', '']:
                    logger.info("Auto format set to ON\n")
                    turn_auto_format_on = True
                    return turn_auto_format_on
                else:
                    logger.info("Auto format set to OFF\n")
                    turn_auto_format_on = False
                    return turn_auto_format_on

        return turn_auto_format_on

    def set_append_option(self):
        append_option = None
        option = yes_no_prompt('Would you like to change the append options?', default_yes=False)
        if option:
            option = self.get_user_append_option()
            return append_option

        return None

    def get_user_append_option(self):
        option = 'not chosen'
        while not 'none' in option or not 'full' in option or not 'name' in option:
            prompt_message = 'Input the append option you would like to use: '
            # Python 2
            confirm = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             option = input(prompt_message)
            # End Python 3
            if self.validate_append_option(option) == True:
                logger.info("Append option set to \'{0}\'\n".format(option))
                return option

    def validate_append_option(self, append_option):
        if append_option in {'none', 'full'} or append_option[:append_option.find(':')+1] in {'number:', 'name:'}:
            if append_option[:3] == 'num':
                try: int(append_option[7:])
                except ValueError:
                    logger.warning('Error: Input after "number" must be an integer')
                    return False
                return True
            elif append_option[:4] == 'name' and len(append_option) <= 5:
                logger.warning('Error: No input given after "name"')
                return False
            else:
                return True
        else:
            logger.warning('Error: Not a valid option')
            return False

    def set_finalized_file_option(self):
        finalized_file = 'off'
        confirm = 'none'
        while confirm.lower() not in ['on', 'off', '']:
            prompt_message = 'Would you like to turn finalized file download on or off? [on/OFF]: '
            # Python 2
            confirm = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             confirm = input(prompt_message)
            # End Python 3
        if confirm.lower() == 'on':
            logger.info("Finalized file download set to ON\n")
            finalized_file = 'on'
        else:
            logger.info("Finalized file download set to OFF\n")
            finalized_file = 'off'

        return finalized_file

    def prompt_unzip_file_option(self):
        unzip_file = 'on'
        confirm = 'none'
        while confirm != 'on' and confirm != 'On' and confirm != 'ON' and confirm != 'off' and confirm != 'Off' and confirm != '':
            prompt_message = 'Would you like to turn finalized file UNZIP on or off? [ON/off]: '
            # Python 2
            confirm = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             confirm = input(prompt_message)
            # End Python 3
            if confirm in ['on', 'On', 'ON', 'off', 'Off', '']:
                if confirm in ['on', 'On', 'ON', '']:
                    logger.info("Finalized file UNZIP set to ON\n")
                    unzip_file = 'on'
                else:
                    logger.info("Finalized file UNZIP set to OFF\n")
                    unzip_file = 'off'

        return unzip_file

    def set_metadata_defaults(self):
        print("Set default metadata to be sent with every document that is uploaded to Lingotek.")
        option = yes_no_prompt('Would you like to set default metadata?', default_yes=True)
        if option:
            Action.default_metadata = {}
            metadata = Action.metadata_wizard(Action, set_defaults=True)
        else:
            metadata = {}
        self.config_parser.set('main', 'default_metadata', json.dumps(metadata))
        logger.info("Default metadata set to {0}".format(metadata))

    def set_metadata_prompt(self):
        print("Toggle an automatic prompt for metadata with every document upload/update.  Turning this on will display a prompt every time an add or push command is run that asks if metadata should be sent, and if it should be sent, it gives the option to define the metadata.  Leaving this off will require a command flag to be used to send metadata.")
        option = yes_no_prompt('Would you like to always prompt for metadata?', default_yes=False)
        if option:
            self.config_parser.set('main', 'metadata_prompt', 'on')
            logger.info("Metadata prompting set to ON")
        else:
            self.config_parser.set('main', 'metadata_prompt', 'off')
            logger.info("Metadata prompting set to OFF")

    def set_metadata_fields(self):
        print("Set the fields that will be displayed when adding, editing, and viewing document metadata.  The possible fields are:\n")
        print(', '.join(str(field) for field in METADATA_FIELDS)+"\n")
        valid = False
        while not valid:
            prompt_message = "Enter a comma-separated list of fields to include, or enter 'all' to include all the fields [all]: "
            # Python 2
            options = raw_input(prompt_message)
            # End Python 2
            # Python 3
#             options = input(prompt_message)
            # End Python 3
            valid, fields = Action.validate_metadata_fields(Action, options)
        self.config_parser.set('main', 'metadata_fields', json.dumps(fields))
        logger.info("Metadata fields set to {0}".format(fields))


from ltk.actions.action import *

class CleanAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def clean_action(self, force, dis_all, path):
        try:
            if dis_all:
                # disassociate everything
                self._clean_all()
                return
            if path:
                locals_to_delete = self._clean_by_path(path)
            else:
                locals_to_delete = self._check_docs_to_clean()

            # remove entry for local doc -- possibly delete local file too?
            if locals_to_delete:
                for curr_id in locals_to_delete:
                    removed_title = self._clean_local(curr_id, force)
                    logger.info('Removing association for document {0}'.format(removed_title))
            else:
                logger.info('Local documents already up-to-date with Lingotek Cloud')
                return
            logger.info('Cleaned up associations between local documents and Lingotek Cloud')
        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on clean: "+str(e))

    def _clean_local(self, doc_id, force):
        removed_doc = self.doc_manager.get_doc_by_prop('id', doc_id)
        if not removed_doc:
            return
        removed_title = removed_doc['name']
        if force:
            self.delete_local(removed_title, doc_id)
        self.doc_manager.remove_element(doc_id)

        return removed_title

    def _clean_by_path(self, path):
        locals_to_delete = []
        files = get_files(path)
        docs = self.doc_manager.get_file_names()
        # Efficiently go through the files in case of a large number of files to check or a large number of remote documents.
        if (files != None) and (len(files) > len(docs)):
            for file_name in docs:
                for name in files:
                    if file_name in name:
                        entry = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(file_name))
                        if entry:
                            locals_to_delete.append(entry['id'])
                            self._cancel_document(entry['id'])
        elif files != None:
            for file_name in files:
                entry = self.doc_manager.get_doc_by_prop('file_name', self.norm_path(file_name))
                if entry:
                    locals_to_delete.append(entry['id'])
                    self._cancel_document(entry['id'])

        return locals_to_delete

    def _check_docs_to_clean(self):
        locals_to_delete = []
        local_ids = self.doc_manager.get_doc_ids()
        for check_id in local_ids:
            response = self.api.document_status(check_id)
            if response.status_code == 200:
                if response.json()['properties']['status'].upper() == 'CANCELLED':
                    locals_to_delete.append(check_id)
            elif response.status_code == 404:
                locals_to_delete.append(check_id)
            else:
                raise_error(response.json(), 'Error trying to list documents in TMS for cleaning')

        # check local files
        db_entries = self.doc_manager.get_all_entries()
        for entry in db_entries:
            # if local file doesn't exist, remove entry
            if not os.path.isfile(os.path.join(self.path, entry['file_name'])):
                locals_to_delete.append(entry['id'])
                self._cancel_document(entry['id'])

        return locals_to_delete

    def _clean_all(self):
        local_ids = self.doc_manager.get_doc_ids()
        for doc_id in local_ids:
            self._cancel_document(doc_id)
        self.doc_manager.clear_all()
        logger.info("Removed all associations between local and remote documents.")

    def _cancel_document(self, cancel_id):
        response = self.api.document_cancel(cancel_id)
        if response.status_code == 404 or response.status_code == 204:
            return
        if response.status_code == 400:
            for message in response.json()['messages']:
                if 'already in a completed state' in message:#currently the API response when cancelling a document that has already been cancelled or completed is "Unable to cancel documents which are already in a completed state.  Current status: COMPLETED/CANCELLED" (COMPLETED or CANCELLED, not COMPLETED/CANCELLED)
                    return
        logger.warning('Error cleaning up association in TMS for document id {0}'.format(doc_id))



from ltk.actions.action import *

class MoveAction(Action):
    def __init__(self, add, path):
        Action.__init__(self, path)
        self.uploadWaitTime = 300
        self.rename = False
        self.source_type = 'file'
        self.path_to_source = ''
        self.path_to_destination = ''
        self.path_sep = ''
        self.directory_to_destination = ''
        self.source = ''
        self.doc = None
        self.folder = None
        self.directory_to_source = ''
        self.add = add

    def mv_file(self, source, destination):
        self.source = source
        try:
            self.path_sep = os.sep
            self.path_to_source = os.path.abspath(self.source)
            if not self.rename:
                self.path_to_destination = os.path.abspath(destination)
            else:
                self.path_to_destination = self.path_to_source.rstrip(os.path.basename(self.path_to_source))+destination
            repo_directory = self.path_to_source
            while repo_directory and repo_directory != "" and not (os.path.isdir(repo_directory + os.sep+".ltk")):
                repo_directory = repo_directory.split(self.path_sep)[:-1]
                repo_directory = self.path_sep.join(repo_directory)
            if repo_directory not in self.path_to_source or repo_directory not in self.path_to_destination:
                logger.error("Error: Operations can only be performed inside ltk directory.")
                return False
            self.directory_to_source = (self.path_to_source.replace(repo_directory, '',1)).lstrip(self.path_sep)
            self.directory_to_destination = (self.path_to_destination.replace(repo_directory, '',1)).lstrip(self.path_sep)
            folder = None
            if self.source_type == 'file':
                self.doc = self.doc_manager.get_doc_by_prop("file_name",self.directory_to_source)
                if not self.doc:
                    logger.error("Error: File has not been added and so can not be moved.")
                    return False
            elif self.source_type == 'folder':
                self.folder = self.folder_manager.get_folder_by_name(self.directory_to_source)
                if not self.folder:
                    logger.warning("Notice: This folder has not been added, though it may be in a directory that has")
            return self._do_move()
        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on moving "+str(self.source)+": "+str(e))

    def _do_move(self):
        try:
            if self.rename and self.source_type == 'file' and self.path_to_source.rstrip(self.path_sep).rstrip(self.doc['name']) != self.path_to_source.rstrip(self.path_sep):
                new_name = os.path.basename(self.path_to_destination)
                response, document_id = self.locked_doc_response_manager(self.api.document_update(self.doc['id'], title=new_name), self.doc['id'], title=new_name)
                # The reason we return here on 402 or after getting a 423 is because that means the API call will definitely fail
                # So we want to prevent the rest of the move action from happening.
                self.doc['id'] = document_id
                if response.status_code == 402 or response.status_code == 423:
                    return False
                self.doc_manager.update_document('name', new_name, self.doc['id'])
            elif not self.rename:
                file_name = os.path.basename(self.path_to_source)
                self.path_to_destination+=self.path_sep+file_name
                self.directory_to_destination+=self.path_sep+file_name
            os.rename(self.path_to_source, self.path_to_destination)
            if self.source_type == 'file':
                self.doc_manager.update_document('file_name', self.directory_to_destination.strip(self.path_sep), self.doc['id'])
            elif self.folder:
                self.folder_manager.remove_element(self.directory_to_source)
                self.folder_manager.add_folder(self.directory_to_destination)
            if self.source_type == 'folder':
                for file_name in self.doc_manager.get_file_names():
                    if file_name.find(self.directory_to_source) == 0:
                        self.doc = self.doc_manager.get_doc_by_prop("file_name",file_name)
                        self.doc_manager.update_document('file_name', file_name.replace(self.directory_to_source, self.directory_to_destination, 1), self.doc['id'])
            try:
                # We place this try catch block down here because if we know that the response was not a 423 or a 402,
                # then we want the rest of the move to take place (the above if/elif/if statements)
                # This is inside of a try block in the case that response is undefined
                if response.status_code == 202 and 'next_document_id' in response.json():
                    self.doc = self.doc_manager.get_doc_by_prop('id', self.doc['id'])
                    self.doc_manager.update_document('id', response.json()['next_document_id'], self.doc['id'])
                elif response.status_code == 410:
                    self.doc = self.doc_manager.get_doc_by_prop('id', self.doc['id'])
                    doc_name = self.doc['name']
                    self.doc_manager.remove_element(self.doc['id'])
                    print("Document was uploaded, but ID has been archived. Renaming and reuploading")
                    self.add_document(self.path_to_destination, doc_name)
                else:
                    return False
            except Exception:
                response = None
            return True
        except Exception as e:
            log_error(self.error_file_name, e)
            logger.error("Error: "+str(e))
            logger.error("An error prevented document {0} from being moved".format(self.source))
            return False

    def mv_action(self, sources, destination):
        try:
            for source in sources:
                if os.path.isdir(source):
                    if os.path.isdir(destination):
                        self.source_type = 'folder'
                        if self.mv_file(source, destination):
                            logger.info("Folder "+source+" has been moved to "+destination)
                        else: logger.error("Failed to move folder "+source)
                    elif os.path.isfile(destination):
                        logger.error("mv: cannot overwrite non-directory \'"+source+"\' with directory \'"+destination+"\'")
                    else:
                        self.rename = True
                        self.source_type = 'folder'
                        if self.mv_file(source, destination):
                            logger.info("Folder "+source+" has been renamed to "+destination)
                        else: logger.error("Failed to move folder "+source)
                elif os.path.isfile(source):
                    self.source_type = 'file'
                    if os.path.isdir(destination):
                        if self.mv_file(source, destination):
                            logger.info(source+" has been moved to "+destination)
                        else: logger.error("Failed to move file "+source)
                    elif os.path.isfile(destination):
                        self.rename = True
                        if self.mv_file(source, destination):
                            logger.info(source+" has been renamed as "+destination)
                            logger.info(destination+" has been deleted")
                        else: logger.error("Failed to move file "+source)
                    else:
                        self.rename = True
                        if self.mv_file(source, destination):
                            logger.info(source+" has been renamed to "+destination)
                        else: logger.error("Failed to move file "+source)
                else:
                    logger.error("Error: Source file does not exist")
        except Exception as e:
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on mv: "+str(e))



from ltk.actions.action import *

class ImportAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)
        self.skip_ids = []

    def _get_import_ids(self,info):
        mapper = choice_mapper(info)
        chosen_ids = []
        while not len(chosen_ids) > 0:
            prompt_message = 'Documents to import: (Separate indices by comma) '
            # Python 2
            # choice = raw_input(prompt_message)
            # End Python 2
            # Python 3
            choice = input(prompt_message)
            # End Python 3
            try:
                chosen_ids = [list(mapper[int(index)].keys())[0] for index in choice.split(',')]
            except ValueError:
                print ('Some unexpected, non-integer value was included')
        return chosen_ids

    def import_action(self, import_all, force, path, track, no_cancel, ids_to_import=None):
        try:
            path = self.norm_path(path)
            response = self.api.list_documents(self.project_id)
            tms_doc_info = {}
            if response.status_code == 200:
                tms_documents = response.json()['entities']
                for entity in tms_documents:
                    doc_info = {'title': entity['properties']['title'], 'extension': entity['properties']['extension']}
                    if no_cancel or track:
                        statuscheck = self.api.document_status(entity['properties']['id'])
                        if statuscheck.json()['properties']['status'].upper() == 'CANCELLED':
                            if no_cancel:
                                continue
                            self.skip_ids.append(entity['properties']['id'])
                    tms_doc_info[entity['properties']['id']] = doc_info
            elif response.status_code == 204:
                logger.error('No documents to import!')
                return
            else:
                raise_error(response.json(), 'Error finding current documents in Lingotek Cloud')
            if len(tms_doc_info) == 0:
                logger.error('No documents to import!')
                return

            if not ids_to_import:
                if import_all:
                    # Python 2
                    # ids_to_import = tms_doc_info.iterkeys()
                    # End Python 2
                    # Python 3
                    ids_to_import = iter(tms_doc_info)
                    # End Python 3
                else:
                    import_doc_info = {}
                    # Python 2
                    # for k, v in tms_doc_info.iteritems():
                    # End Python 2
                    # Python 3
                    for k, v in tms_doc_info.items():
                    # End Python 3
                        import_doc_info[k] = v['title']
                    ids_to_import = self._get_import_ids(import_doc_info)
            else:
                ids_to_import = [ids_to_import]
            for curr_id in ids_to_import:
                self.import_document(curr_id, tms_doc_info[curr_id], force, path, track)
        except Exception as e:
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on import: "+str(e))

    def import_check(self, document_id, title, force=False, path=False):
        if not path:
            path = self.path
        else:
            path = os.path.join(self.path, path.replace(self.path, ''))
        path_changed = False
        curr_path = False
        write_file = True
        curr_entry = self.doc_manager.get_doc_by_prop('id', document_id)
        new_path = os.path.join(path, title)
        delete_file = False
        if curr_entry:
            curr_path = os.path.join(self.path, curr_entry['file_name'])
            # print (curr_path, new_path)
            if self.norm_path(curr_path) != self.norm_path(new_path):
                path_changed = curr_path
        if not force:
            if not curr_path and not os.path.exists(new_path):
                return path_changed, new_path, write_file, delete_file
            if path_changed and curr_path: # Confirm changing the file path saved in docs.json
                change_option = yes_no_prompt('Would you like to change ' \
                    'the current saved path of '+title+' from '+curr_path+' to '+new_path+'?', default_yes=False)
                if change_option:
                    delete_option = yes_no_prompt('Delete '+curr_path+'?', default_yes=False)
                    if delete_option:
                        delete_file = True
                else:
                    logger.info('Retaining old path "{0}"'.format(curr_path))
                    path_changed = False
                    new_path = curr_path
            # Confirm overwriting a local file
            if os.path.exists(new_path):
                overwrite_option = yes_no_prompt('Would you like to overwrite the existing document at '+new_path+'?', default_yes=False)
                if not overwrite_option:
                    logger.info('Skipped importing "{0}"'.format(title))
                    write_file = False
        # print(str(path_changed)+" "+str(new_path)+" "+str(write_file)+" "+str(delete_file))
        return path_changed, new_path, write_file, delete_file

    def import_document(self, document_id, document_info, force=False, path=False, track=False):
        local_ids = self.doc_manager.get_doc_ids()
        response = self.api.document_content(document_id, None, None, finalized_file=self.finalized_file)
        if(response.status_code == 400):
            if document_info and document_info['title']:
                title = document_info['title']
                logger.info('Failed to import document: {0}'.format(title))
            else:
                logger.info('Failed to import document: {0}'.format(document_id))
            return
        elif (response.status_code == 404):
            if document_info and document_info['title']:
                title = document_info['title']
                logger.info('Document not found {0}'.format(title))
            else:
                logger.info('Document with ID: \'{0}\' not found'.format(document_id))
            return
        title, extension = os.path.splitext(document_info['title'])
        if not extension:
            extension = document_info['extension']
            extension = '.' + extension
        if extension and extension != '.none':
            title += extension
        if path:
            file_path = os.path.join(self.path, path, title)
        else:
            file_path = os.path.join(self.path, title)
        # file_path = os.path.join(os.getcwd(), title)  # import to current working directory
        logger.info('Importing "{0}" to {1}'.format(title, file_path))
        # use status action to get locale info for importing
        try:
            locale_map = self.import_locale_info(document_id)
            # Python 2
            # locale_info = list(locale_map.iterkeys())
            # End Python 2
            # Python 3
            locale_info = list(iter(locale_map))
            # End Python 3
        except exceptions.RequestFailedError:
            print("failed on locale info")
            locale_info = []

        changed_path = False
        changed_path, new_path, write_file, delete_file = self.import_check(document_id, title, force, path)
        if delete_file and changed_path and os.path.exists(changed_path):
            self.delete_local_path(changed_path, 'Deleting local file {0}'.format(changed_path))

        if write_file:
            try:
                with open(new_path, 'wb') as fh:
                    for chunk in response.iter_content(1024):
                        fh.write(chunk)
            except IOError as e:
                print(e.errno)
                print(e)
        new_path = self.norm_path(new_path)
        if track:
            if document_id in self.skip_ids:
                return
            if document_id not in local_ids:
                self._add_document(new_path, title, document_id, 'imported')
                self.doc_manager.update_document('locales', locale_info, document_id)
            elif changed_path:
                # update the document's path
                logger.info('Moved local file {0} to {1}'.format(changed_path, new_path))
                self.doc_manager.update_document('file_name', new_path, document_id)



from ltk.actions.action import *
import ctypes
import socket
import ltk.check_connection

class AddAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def add_action(self, file_patterns, **kwargs):
        # format will be automatically detected by extension but may not be what user expects
        # use current working directory as root for files instead of project root

        try:
            # add folders to the db
            added_folder = self.add_folders(file_patterns)
            if 'directory' in kwargs and kwargs['directory']:
                if not added_folder:
                    logger.info("No folders to add at the given path(s).")
                return

            # get files to add and check if they exist
            matched_files = get_files(file_patterns)
            if not matched_files:
                if added_folder:
                    # folders have been added and there are no documents to add, nothing left to do
                    return
                else:
                    raise exceptions.ResourceNotFound("Could not find the specified file/pattern.")

            # add the documents to the db and lingotek cloud
            self.add_documents(matched_files, **kwargs)

        except Exception as e:
            log_error(self.error_file_name, e)
            if 'string indices must be integers' in str(e) or 'Expecting value: line 1 column 1' in str(e):
                logger.error("Error connecting to Lingotek's TMS")
            else:
                logger.error("Error on add: "+str(e))

    def add_documents(self, matched_files, **kwargs):
        ''' adds new documents to the lingotek cloud and, after prompting user, overwrites changed documents that
                have already been added '''
        metadata = copy.deepcopy(self.default_metadata)
        if 'metadata' in kwargs and kwargs['metadata']:
            metadata = self.metadata_wizard()
        elif self.metadata_prompt:
            if yes_no_prompt('Would you like to launch the metadata wizard?', default_yes=True):
                metadata = self.metadata_wizard()
        confirmed=False
        for file_name in matched_files:
            try:
                # title = os.path.basename(os.path.normpath(file_name)).split('.')[0]
                relative_path = self.norm_path(file_name)
                title = os.path.basename(relative_path)
                try:
                    if os.stat(os.path.join(self.path,relative_path)).st_size == 0:
                        logger.info("This document is empty and was not added: {0}\n".format(title))
                        continue
                except FileNotFoundError:
                    logger.warning("Warning: could not verify that {0} is not empty".format(relative_path))
                except Exception as e:
                    raise e
                if not self.doc_manager.is_doc_new(relative_path):
                    if self.doc_manager.is_doc_modified(relative_path, self.path) or len(metadata) > 0:
                        if 'overwrite' in kwargs and kwargs['overwrite']:
                            confirmed = True
                        try:
                            if not confirmed:
                                option = yes_no_prompt("Document \'{0}\' already exists. Would you like to overwrite it?".format(title), default_yes=False)
                            else:
                                option = True

                            # confirm if would like to overwrite existing document in Lingotek Cloud
                            if option:
                                logger.info('Overwriting document \'{0}\' in Lingotek Cloud...\n'.format(title))
                                self.update_document_action(file_name, title, doc_metadata=metadata, **kwargs)
                                continue
                            else:
                                logger.info('Will not overwrite document \'{0}\' in Lingotek Cloud\n'.format(title))
                                continue
                                
                        except KeyboardInterrupt:
                            logger.error("Canceled adding the document")
                            return
                    else:
                        logger.error("This document has already been added and no metadata is being sent: {0}\n".format(title))
                        continue
            except json.decoder.JSONDecodeError:
                logger.error("JSON error on adding document.")
            except Exception:
                logger.error("Error adding document")
            else:
                self.add_document(file_name, title, doc_metadata=metadata, **kwargs)

    def add_folders(self, file_patterns):
        ''' checks each file pattern for a directory and adds matching patterns to the db '''
        ''' returns true if folder(s) have been added, otherwise false '''

        added_folder = False
        for pattern in file_patterns:
            if os.path.exists(pattern):
                if os.path.isdir(pattern):
                    if self.is_hidden_file(pattern):
                        logger.warning("Folder is hidden")
                    # elif self.download_dir != None and os.path.samefile(pattern, self.download_dir):
                    #     logger.warning("The folder " + "\'" + pattern + "\'" + " set as the downlaod folder. You cannot add the download folder. To remove downlaod folder use \'ltk config -d --none\' command.")
                    elif not self._is_folder_added(pattern):
                        self.folder_manager.add_folder(self.norm_path(pattern.rstrip(os.sep)))
                        logger.info("Added folder "+str(pattern))
                    else:
                        logger.warning("Folder "+str(pattern)+" has already been added.\n")
                    added_folder = True
            else:
                logger.warning("Path \""+str(pattern)+"\" doesn't exist.\n")

        return added_folder

    def _is_folder_added(self, file_name):
        """ checks if a folder has been added or is a subfolder of an added folder """
        folder_names = self.folder_manager.get_file_names()
        for folder in folder_names:
            # print("folder: "+str(os.path.join(self.path,folder)))
            # print("folder to be added: "+os.path.abspath(file_name))
            if os.path.join(self.path,folder) in os.path.abspath(file_name):
                return True
        return False


from ltk.actions.action import *

class FiltersAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def filter_list_action(self):
        response = self.api.list_filters()
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get filters')
        filter_entities = response.json()['entities']
        table = []
        headers = ["Title","Created","ID"]
        for entry in sorted(filter_entities, key=lambda entry: entry['properties']['upload_date'], reverse=True):
            properties = entry['properties']
            filter_id = properties['id']
            upload_date = time.strftime("%Y-%m-%d", time.localtime(int(properties['upload_date']/1000)))
            is_public = " (public)" if properties['is_public'] else ""
            title = "{0}{1}".format(properties['title'], is_public)
            table.append([title,upload_date,filter_id])
        print(tabulate(table,headers=headers))

    def filter_rm_action(self, filter_id):
        response = self.api.delete_filter(filter_id)
        if response.status_code != 204:
            raise_error(response.json(), 'Failed to remove filter {0}'.format(filter_id), filter_id)
        else:
            print('Filter {0} was successfully removed.'.format(filter_id))
        return

    def filter_add_action(self, filename, filter_type):
        if os.path.isfile(filename):
            content = open(filename, 'rb')
            response = self.api.post_filter(filename, filter_type)
            if response.status_code != 201:
                raise_error(response.json(), 'Failed to create filter', filename)
            else:
                print('Filter {0} was successfully added.'.format(response.json()['properties']['id']))
        else:
            print("File \"{0}\" not found.".format(filename))

    def filter_save_action(self, filter_id, filename):
        if os.path.isfile(filename):
            content = open(filename, 'rb')
            response = self.api.patch_filter(filter_id, filename)

            if response.status_code != 202:
                raise_error(response.json(), 'Failed to save filter', filename)
            else:
                print('Filter {0} was successfully updated.'.format(filter_id))
        else:
            print("File \"{0}\" not found.".format(filename))

    def filter_get_action(self, filter_id, filename, overwrite=False):
        response = self.api.get_filter_content(filter_id)
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get filter', filter_id, filename)
            return
        if filename is None:
            import cgi
            value, params = cgi.parse_header(response.headers['Content-Disposition'])
            filename = params['filename']
        if os.path.exists(filename) and overwrite != True:
            option = yes_no_prompt('Filter "{0}" already exists locally. Would you like to overwrite it?'.format(filename), default_yes=False)
            if not option:
                logger.info('Will not overwrite local filter "{0}"'.format(filename))
                return
        try: # save filter content to filename
            with open(filename, 'wb') as fh:
                fh.write(response.content)
                logger.info('Filter {0} saved to local file "{1}"'.format(filter_id,filename))
        except:
            print("Unexpected error:", sys.exc_info()[0])
            raise


    def filter_info_action(self, filter_id):
        response = self.api.get_filter_info(filter_id)
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get filter', filter_id)
            return
        if 'properties' in response.json():
            for key in response.json()['properties']:
                print ('{0:15} {1}'.format(key + ':', response.json()['properties'][key]))
        return



# Python 2
from ConfigParser import ConfigParser, NoOptionError
# End Python 2
# Python 3
# from configparser import ConfigParser, NoOptionError
# End Python 3
from ltk.constants import CONF_DIR, CONF_FN, SYSTEM_FILE, ERROR_FN
import sys
import os.path
import codecs
from git import Repo
from git import RemoteProgress
import os
try:
	import pexpect
except: pass
import binascii
from ltk.utils import error

class Git_Auto:
	def __init__(self, path):
		self.path = path
		self.join = os.path.join
		self.repo_is_defined = False
		self.repo_directory = ""
		self.config_file_name, self.conf_parser = self.init_config_file()

	def init_config_file(self):
		config_file_name = os.path.join(self.path, CONF_DIR, CONF_FN)
		conf_parser = ConfigParser()
		conf_parser.read(config_file_name)
		return config_file_name, conf_parser

	def repo_exists(self, repo_directory=os.getcwd()):
		while repo_directory and repo_directory != "" and not (os.path.isdir(repo_directory + os.sep+".git")):
			repo_directory = repo_directory.split(os.sep)[:-1]
			repo_directory = (os.sep).join(repo_directory)
		if not repo_directory or repo_directory == "":
			error("No Git repository found for the current directory.")
			return False
		else:
			self.repo_directory = repo_directory
			self.initialize_repo()
			return True

	def initialize_repo(self, directory=None):
		if not directory:
			self.repo = Repo(self.repo_directory)
		else:
			self.repo = Repo(directory)
		self.repo_is_defined = True

	def add_file(self, file_name):
		if self.repo_is_defined:
			if(self.repo_exists()):
				self.repo.git.add(file_name)

	def commit(self, message):
		if self.repo_is_defined:
			if(self.repo_exists()):
				message.rstrip(' ')
				self.repo.index.commit(message)

	def encrypt(self, password):
		# Python 2
		password = codecs.encode(password, 'base64')
		# End Python 2
		# Python 3
# 		password = bytes(password, 'utf-8')
# 		password = codecs.encode(password, 'base64')
# 		password = str(password, 'utf-8')
		# End Python 3
		return password

	def push(self, username=None, password=None):
		username = self.conf_parser.get('main', 'git_username')
		password = self.conf_parser.get('main', 'git_password')
		if not self.repo_is_defined:
			if not (self.repo_exists()):
				error("No Git repository found for the current directory.")
				return
		try:
			g = pexpect.spawnu('git push')
		except:
			try:
				self.repo.git.push()
				print("Push was successful")
			except Exception as e:
				error("Git push failed!")
				print(type(e))
			return
		try:
			g.logfile_read = sys.stdout
			while True:
				i = g.expect([u'Username for .*', u'Password for .*', pexpect.EOF])
				if(i == 0):
					g.send(username+'\n')
				elif(i == 1):
					# Python 2
					g.send(codecs.decode(password, 'base64')+'\n')
					# End Python 2
					# Python 3
# 					g.send(str(codecs.decode(password.encode(), 'base64'), 'utf-8')+'\n')
					# End Python 3
				else:
					break
		except Exception as e:
			print("Notice: Auto-credentials currently not operational")
		'''
		# if 'nt' not in os.name:
		# 	g = pexpect.spawnu('git push')
		# else:
		# 	try:
		# 		g = winpexpect.winspawn('git push')
		# 	except:
		# 		error("Push failed! Please confirm that the winpexpect module is up to date.")
		# 		return
		# g.logfile_read = sys.stdout
		# if not username: username = ''
		# if not password: password = ''
		# while True:
		# 	i = g.expect([u'Username for .*', u'Password for .*', pexpect.EOF])
		# 	if(i == 0):
		# 		g.send(username+'\n')
		# 	elif(i == 1):
		# 		# Python 2
				g.send(codecs.decode(password, 'base64')+'\n')
		# 		# End Python 2
  		#		# Python 3
# 		# 		g.send(str(codecs.decode(password.encode(), 'base64'), 'utf-8')+'\n')
		# 		# End Python 3
		# 	else:
		# 		break
		'''



from tinydb import TinyDB, where
import os
import requests
import json
from ltk.constants import CONF_DIR, DB_FN, FOLDER_DB_FN
from ltk.apicalls import ApiCalls

# Python 2
from ConfigParser import ConfigParser, NoOptionError
# End Python 2
# Python 3
# from configparser import ConfigParser, NoOptionError
# End Python 3

class DocumentManager:
    def __init__(self, path):
        self.db_file = os.path.join(path, CONF_DIR, DB_FN)
        self._db = TinyDB(self.db_file)

    def open_db(self):
        self._db = TinyDB(self.db_file)

    def close_db(self):
        self._db.close()

    def doc_exists(self, file_name, title):
        entries = self._db.search((where('file_name') == file_name) & (where('name') == title))
        if entries:
            return True
        else:
            return False

    def is_doc_new(self, file_name, root_path=None):
        if root_path != None:
            config = ConfigParser()
            try:
                config.read( root_path + os.sep+".ltk"+os.sep+"config")
                download_folder = config.get('main','download_folder')
                locale_folders = config.get('main','locale_folders')
                locale_folders = json.loads(locale_folders)
            except NoSectionError as e:
                return False
            #if doc is in the user specified download folder return false
            download_is_watched = False

            ### Conditions the if statement is checking:
            #       Is the file_name and download folder the same location? (Not good)
            #       Make sure download folder is not the empty string in the case that the first check passes
            #       Is the file name inside of the download folder (it's in a sub-folder of the download folder)? (That's okay)
            ###
            if file_name[:len(download_folder)] == download_folder and download_folder is not "" and len(file_name) < len(download_folder) :
                with open(root_path + os.sep+'.ltk'+os.sep+'folders.json') as json_data:
                    folders_json = json.load(json_data)
                folders = folders_json['_default']
                for folder in folders:
                    name = folders_json['_default'][folder]['file_name']
                    if name == download_folder:
                        download_is_watched = True

                if download_is_watched == False:
                    return False
            #if doc is in a user specified locale download folder return false
            for k,v in locale_folders.items():
                if file_name[:len(v)] == v:
                    return False
            #if doc is in a clone folder of a locale name return false
            content = self._db.all()
            for doc in content:
                if 'file_name' in doc.keys():
                    name = doc.get('file_name')
                if 'downloaded' in doc.keys():
                    locales = doc.get('downloaded')
                    name_parts = name.strip().split(".")
                    file_name_name = ""
                    for x in range(0, len(name_parts)-1):
                        file_name_name += name_parts[x]
                    for locale in locales:
                        possible_no_clone_file_name = file_name_name + "." + locale + "." + name_parts[len(name_parts)-1]
                        if download_is_watched:
                            possible_file_name = os.path.join(download_folder, os.path.join(locale, (os.path.basename(file_name_name) + "." + name_parts[len(name_parts)-1])))
                        else:
                            possible_file_name = os.path.join(locale, name)
                        possible_file_name = possible_file_name.replace("_","-")
                        if possible_file_name == file_name or possible_no_clone_file_name == file_name:
                            return False

        file_name_exists = self._db.search(where('file_name') == file_name)
        if not file_name_exists:
            return True
        return False

    ''' receives a translation file and checks if there are corresponding source files'''
    def is_translation(self, file_name, title, matched_files, actions):
        ''' check if the file is a translation file'''

        for myFile in matched_files:
            relative_path = actions.norm_path(myFile)
            myFileTitle = os.path.basename(relative_path)

            ''' only compare the file being checked against source files that have already been added '''
            entry = self._db.get(where("file_name") == relative_path)
            if entry:
                ''' check the source file's download codes to see if the file being checked is a translation file '''
                downloads = self.get_doc_downloads(relative_path)
                if downloads:
                    for d in downloads:
                        ''' append the download code to the source file for comparison '''
                        temp = myFileTitle.split(".")
                        newString = temp[0]+"."+ d +"."+temp[1]
                        if newString == title:
                            return True

        return False

    ''' receives a source file and finds the source files associated with it '''
    #def delete_local_translations(self, file_name, path, actions):


    def is_doc_modified(self, file_name, path):
        entry = self._db.get(where('file_name') == file_name)
        full_path = os.path.join(path, file_name)
        last_modified = os.stat(full_path).st_mtime
        if entry and entry['added'] < last_modified and entry['last_mod'] < last_modified:
            return True
        return False

    def add_document(self, title, create_date, doc_id, sys_mtime, last_mod, file_name, process_id, download_folder=''):
        entry = {'name': title, 'added': create_date, 'id': doc_id,
                 'sys_last_mod': sys_mtime, 'last_mod': last_mod, 'file_name': file_name,
                 'downloaded': [], 'download_folder': download_folder, 'process_id': process_id}
        self._db.insert(entry)

    def update_document(self, field, new_val, doc_id):
        if type(new_val) is list:
            self._db.update(_update_entry_list(field, new_val), where('id') == doc_id)
        else:
            if type(new_val) is set:
                new_val = list(new_val)
            self._db.update({field: new_val}, where('id') == doc_id)

    def get_doc_by_prop(self, prop, expected_value):
        """ get documents by the specified property """
        entry = self._db.get(where(prop) == expected_value)
        return entry

    def get_all_entries(self):
        return self._db.all()

    def get_doc_ids(self):
        """ returns all the ids of documents that the user has added """
        doc_ids = []
        for entry in self._db.all():
            doc_ids.append(entry['id'])
        return doc_ids

    def get_process_ids(self):
        """ returns all the process ids of documents that the user has added"""
        process_ids = []
        for entry in self._db.all():
            if 'process_id' in entry:
                process_ids.append(entry['process_id'])
        return process_ids

    def get_file_names(self):
        """ returns all the file names of documents that the user has added """
        file_names = []
        for entry in self._db.all():
            file_names.append(entry['file_name'])
        return file_names

    def get_names(self):
        """ returns all the names of documents that the user has added """
        file_names = []
        for entry in self._db.all():
            file_names.append(entry['name'])
        return file_names

    def get_doc_name(self, file_name):
        """ returns the file name of a document for a given file path """
        entry = self._db.get(where("file_name") == file_name)
        if entry:
            return entry['name']
        else:
            return None

    def get_doc_locales(self, file_name):
        """ returns the target locales of a document for a given file """
        locales = []
        entry = self._db.get(where("file_name") == file_name)
        if entry:
            locales.append(entry['locales'])

        return locales

    def get_doc_downloads(self, file_name):
        """ returns all the downloaded translations for a given file """
        entry = self._db.get(where("file_name") == file_name)
        if entry:
            downloads = entry['downloaded']
            return downloads

    def get_doc_target_folder(self, file_name):
        """ returns the target download folder for a given file """
        folder = ''
        entry = self._db.get(where("file_name") == file_name)
        if entry:
            if 'download_folder' not in entry:
                entry['download_folder'] = ''
            folder = entry['download_folder']
        return folder

    def remove_element(self, doc_id):
        self._db.remove(where('id') == doc_id)

    def clear_prop(self, doc_id, prop):
        """ Clear specified property of a document according to its type """
        entry = self._db.get(where('id') == doc_id)
        if isinstance(entry[prop],str):
            self.update_document(prop,"",doc_id)
        elif isinstance(entry[prop],int):
            self.update_document(prop,0,doc_id)
        elif isinstance(entry[prop],list):
            self.update_document(prop,[],doc_id)
        elif isinstance(entry[prop],dict):
            self.update_document(prop,{},doc_id)

    def remove_element_in_prop(self, doc_id, prop, element):
        doc_prop = self.get_doc_by_prop('id', doc_id)[prop]
        if element in doc_prop:
            doc_prop.remove(element)
        self.update_document(prop, doc_prop, doc_id)

    def add_element_to_prop(self, doc_id, prop, element):
        doc_prop = self.get_doc_by_prop('id',doc_id)[prop]
        if element not in doc_prop:
            doc_prop.append(element)
        self.update_document(prop, doc_prop, doc_id)

    def clear_all(self):
        self._db.purge()

def _update_entry_list(field, new_val):
    """ updates a list in an entry """
    def transform(element):
        try:
            element[field]
        except KeyError:
            element[field] = []
        if new_val:
            # element[field].extend(new_val)
            for i in range(len(new_val)):
                new_val[i] = new_val[i].replace('-', '_')
            # element[field].extend([val.replace('-', '_') for val in new_val])
            # element[field] = list(set(element[field]))
            element[field] = new_val
        else:
            element[field] = []

    return transform

class FolderManager:
    def __init__(self, path):
        self.db_file = os.path.join(path, CONF_DIR, FOLDER_DB_FN)
        self._db = TinyDB(self.db_file)

    def open_db(self):
        self._db = TinyDB(self.db_file)

    def close_db(self):
        self._db.close()

    def add_folder(self, file_name):
        if not self.folder_exists(file_name):
            entry = {'file_name': file_name}
            self._db.insert(entry)

    def get_all_entries(self):
        return self._db.all()

    def folder_exists(self, file_name):
        """ checks if a folder has been added """
        entries = self._db.search(where('file_name') == file_name)
        if entries:
            return True
        else:
            return False

    def remove_element(self, file_name):
        self._db.remove(where('file_name') == file_name)

    def get_file_names(self):
        """ returns all the file names of folders that the user has added """
        file_names = []
        for entry in self._db.all():
            file_names.append(entry['file_name'])
        return file_names

    def get_folder_by_name(self, expected_name):
        """ get documents by the specified property """
        entry = self._db.get(where('file_name') == expected_name)
        return entry

    def clear_all(self):
        self._db.purge()



import os
import shutil
import fnmatch

def get_files(patterns):
    """ gets all files matching pattern from root
        pattern supports any unix shell-style wildcards (not same as RE) """

    cwd = os.getcwd()
    if isinstance(patterns,str):
        patterns = [patterns]

    matched_files = []
    for pattern in patterns:
        path = os.path.abspath(pattern)
        # print("looking at path "+str(path))
        # check if pattern contains subdirectory
        if os.path.exists(path):
            if os.path.isdir(path):
                for root, subdirs, files in os.walk(path):
                    split_path = root.split('/')
                    for file in files:
                        # print(os.path.join(root, file))
                        if fnmatch.fnmatch(file, '*.py'):
                            matched_files.append(os.path.join(root, file))
            else:
                matched_files.append(path)
        else:
            logger.info("File not found: "+pattern)
    if len(matched_files) == 0:
        return None
    return matched_files

dir2 = 'python2/ltk'
files2 = get_files(dir2)

# Copy files from 2 to 3
for fpath2 in files2:
    fpath3 = fpath2.replace('python2','python3')
    shutil.copyfile(fpath2, fpath3)

# Comment and uncomment specified lines in Python 3 version
for fpath in files2:
    fpath = fpath.replace('python2','python3')
    with open(fpath, 'r+') as f:
        lines = f.readlines()
        f.seek(0)
        f.truncate()
        is_python3 = False
        is_python2 = False
        for line in lines:
            if '# Python 3' in line:
                is_python3 = True
            elif is_python3:
                if '# End Python 3' in line:
                    is_python3 = False
                    continue
                line = line.replace('# ','')
            elif '# Python 2' in line:
                is_python2 = True
            elif is_python2:
                if '# End Python 2' in line:
                    is_python2 = False
                    continue
                line = '# '+str(line)
            f.write(line)


from ltk.actions.action import *

class CloneAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)

    def clone_folders(self, dest_path, folders_map, locale, copy_root=False):
        """ Copies subfolders of added folders to a particular destination folder (for a particular locale).
            If there is more than one root folder to copy, each root folder is created inside of the destination folder.
            If there is only one root folder to copy, only the subdirectories are copied."""

        try:
            # print("dest_path: "+str(dest_path))
            # print("folders to clone: "+str(folders_map))
            if not folders_map or not len(folders_map):
                logger.warning("No folders to clone for locale "+str(locale)+".")
                return
            folder_created = False
            prefix_folder = False
            if not os.path.exists(dest_path):
                os.mkdir(dest_path)
                folder_created = True
            if len(folders_map) > 1 or copy_root:
                prefix_folder = True
            for root_folder in folders_map:
                # print("root folder: "+root_folder)
                if prefix_folder:
                    new_root_path = dest_path + os.sep + root_folder
                    if not os.path.exists(new_root_path):
                        os.mkdir(new_root_path)
                        folder_created = True
                    for folder in folders_map[root_folder]:
                        new_sub_root_path = dest_path + os.sep + root_folder + os.sep + folder
                        if not os.path.exists(new_sub_root_path):
                            os.mkdir(new_sub_root_path)
                            folder_created = True
                            # print("created folder "+new_sub_root_path)
                else:
                    if folders_map[root_folder]:
                        for folder in folders_map[root_folder]:
                            new_path = dest_path + os.sep + folder
                            if not os.path.exists(new_path):
                                os.mkdir(new_path)
                                folder_created = True
        except IOError as e:
            print(e.errno)
            print(e)
            if folder_created != True:
                folder_created = False
                                    # print("created folder "+ new_path)
        return folder_created

    def clone_action(self, folders, copy_root):
        try:
            if not len(self.watch_locales) or self.watch_locales == set(['[]']):
                logger.warning("There are no locales for which to clone. You can add locales using 'ltk config -t'.")
                return
            folders_map = {}
            are_added_folders = False
            if not folders:
                folders = self.folder_manager.get_file_names()
                are_added_folders = True
            for folder in folders:
                folder_paths = folder.split(os.sep)
                # print("current abs: "+str(self.get_current_abs(folder)))
                if are_added_folders:
                    folder = os.path.join(self.path,folder)
                # print("folder to be cloned: "+str(folder))
                folders_map[folder_paths[len(folder_paths)-1]] = self.get_sub_folders(folder)
            # print("folders: "+str(folders_map))
            cloned_folders = False
            for locale in self.watch_locales:
                dest_path = ""
                if locale in self.locale_folders:
                    dest_path = self.locale_folders[locale]
                else:
                    if self.download_dir and self.download_dir != 'null':
                        dest_path = os.path.join(os.path.join(self.path,self.download_dir),locale)
                    else:
                        dest_path = os.path.join(self.path,locale)
                dest_path = os.path.join(self.path,dest_path)
                if self.clone_folders(dest_path, folders_map, locale, copy_root):
                    logger.info("Cloned locale " + str(locale) + " at " + dest_path)
                    cloned_folders = True
            if not len(self.folder_manager.get_file_names()):
                logger.warning("There are no added folders to clone.")
                return
            if not cloned_folders:
                logger.info("All locales have already been cloned.")
            return
        except Exception as e:
            log_error(self.error_file_name, e)
            logger.error("Error on clean: "+str(e))


    def get_sub_folders(self, patterns):
        """ gets all sub-folders matching pattern from root
            pattern supports any unix shell-style wildcards (not same as RE)
            returns the relative paths starting from each pattern"""

        cwd = os.getcwd()
        if isinstance(patterns,str):
            patterns = [patterns]
        allPatterns = []
        if isinstance(patterns,list) or isinstance(patterns,tuple):
            for pattern in patterns:
                # print("pattern in loop: "+str(pattern))
                basename = os.path.basename(pattern)
                if basename and basename != "":
                    allPatterns.extend(self.getRegexDirs(pattern,cwd))
                else:
                    allPatterns.append(pattern)
        else:
            basename = os.path.basename(patterns)
            if basename and basename != "":
                allPatterns.extend(self.getRegexDirs(patterns,cwd))
            else:
                allPatterns.append(patterns)
        matched_dirs = []
        # print("all patterns: "+str(allPatterns))
        for pattern in allPatterns:
            path = os.path.abspath(pattern)
            # print("looking at path "+str(path))
            # check if pattern contains subdirectory
            if os.path.exists(path):
                if os.path.isdir(path):
                    for root, subdirs, files in os.walk(path):
                        split_path = root.split(os.sep)
                        for subdir in subdirs:
                            # print(os.path.join(root, subdir))
                            matched_dirs.append(os.path.join(root,subdir).replace(str(path)+os.sep,""))
            else:
                logger.info("Directory not found: "+pattern)
        if len(matched_dirs) == 0:
            return None
        return matched_dirs

    def getRegexDirs(self, pattern,path):
        dir_name = os.path.dirname(pattern)
        if dir_name:
            path = os.path.join(path,dir_name)
        pattern_name = os.path.basename(pattern)
        # print("path: "+path)
        # print("pattern: "+str(pattern))
        matched_dirs = []
        if pattern_name and not "*" in pattern:
            return [pattern]
        for path, subdirs, files in os.walk(path):
            for dn in fnmatch.filter(subdirs, pattern):
                matched_dirs.append(os.path.join(path, dn))
        print("matched dirs: "+str(matched_dirs))
        return matched_dirs



from ltk.actions.action import *
from tabulate import tabulate

class StatusAction(Action):
    def __init__(self, path):
        Action.__init__(self, path)
        self.uploadWaitTime = 300

    def get_status(self, **kwargs):
        try:
            doc_name = None
            detailed = False

            if 'detailed' in kwargs and kwargs['detailed']:
                detailed = True
            if 'doc_name' in kwargs:
                doc_name = kwargs['doc_name']

            if 'all' in kwargs and kwargs['all']:
                self._get_all_status(detailed)
                return
            else:
                doc_ids = self._get_doc_ids(doc_name)
                if not doc_ids:
                    print("No documents to report")
                    return

            for doc_id in doc_ids:
                doc_id = self.get_latest_document_version(doc_id) or doc_id
                self._get_status_of_doc(doc_id, detailed)
        except requests.exceptions.ConnectionError:
            logger.warning("Could not connect to Lingotek")
            exit()
        except ValueError:
            logger.warning("Could not connect to Lingotek")
            exit()
        # Python 3
        #except json.decoder.JSONDecodeError:
            #logger.warning("Could not connect to Lingotek")
            #exit()
        # End Python 3
        except Exception as e:
            log_error(self.error_file_name, e)
            logger.warning("Error on requesting status: "+str(e))

    def _get_doc_ids(self, doc_name):
        if doc_name is not None:
            entry = self.doc_manager.get_doc_by_prop('name', doc_name)
            try:
                doc_ids = [entry['id']]
            except TypeError:
                raise exceptions.ResourceNotFound("Document name specified for status doesn't exist: {0}".format(doc_name))
        else:
            doc_ids = self.doc_manager.get_doc_ids()

        return doc_ids

    def _get_all_status(self, detailed):
        response = self.api.list_documents(self.project_id)
        if response.status_code == 204:
            print("No documents to report")
            return
        elif response.status_code != 200:
            if check_response(response):
                raise_error(response.json(), "Failed to get status of documents", True)
            else:
                raise_error("", "Failed to get status of documents", True)
        else:
            for entry in response.json()['entities']:
                #title = entry['entities'][0]['properties']['title']
                #progress = entry['entities'][0]['properties']['progress']
                #self._print_status(title, progress)
                #if detailed:
                #    self._print_detailed_status(entry['properties']['id'], title)
                self._get_status_of_doc(entry['properties']['id'], detailed)

    def _get_status_of_doc(self, doc_id, detailed):
        doc_id = self.get_latest_document_version(doc_id) or doc_id
        response = self.api.document_status(doc_id)
        if response.status_code != 200:
            entry = self.doc_manager.get_doc_by_prop('id', doc_id)
            if entry:
                error_message = "Failed to get status of document "+entry['file_name']
            else:
                error_message = "Failed to get status of document "+str(doc_id)
            if check_response(response):
                raise_error(response.json(), error_message, True, doc_id)
            else:
                self._get_process(entry)
        else:
            title = response.json()['properties']['title']
            progress = response.json()['properties']['progress']
            statustext = response.json()['properties']['status'].upper()
            self._print_status(title, doc_id, progress, statustext)
            if detailed:
                self._print_detailed_status(doc_id, title)

    def _get_process(self, entry):
        if 'process_id' not in entry:
            error_message = "Check Lingotek TMS to see if \'"+entry['file_name']+"\' has been deleted or was not properly imported"
            raise_error("", "Not Found: "+error_message, True, entry['id'])
            return
        process_id = entry['process_id']
        if process_id == 'imported':#documents added with ltk import -t don't come with a process id, so they get set to 'imported'
            error_message = "Check Lingotek TMS to see if \'"+entry['file_name']+"\' has been deleted"
            raise_error("", "Not Found: "+error_message, True, entry['id'])
            return
        response = self.api.get_process(process_id)
        if response.status_code == 404:
            # The process doesn't exist for some reason
            self._failed_entry(entry['id'], entry['name'])
        else:
            status = response.json()['properties']['status']
            progress = response.json()['properties']['progress']
            if status.lower() == 'in_progress':
                # Process is currently in progress. Replaces need for upload wait time since now we can get the
                # current document process progress.
                print('Uploading document {0}: {1}% complete'.format(entry['name'], progress))
            elif status.lower() == 'completed':
                # Process is completed and the document was uploaded to TMS, but there was an error in getting the document status
                # Seems to happen when the document is deleted from within TMS
                print('Document {0} was imported, but could not be found within TMS. You may need to run ltk clean to update the local database'.format(entry['name']))
            else:
                # Process has a failed status
                self._failed_entry(entry['id'], entry['name'])

    def _failed_entry(self, doc_id, name):
        error_message = "\'"+name+"\' failed to import properly"
        raise_error("", "Not Found: "+error_message, True, doc_id)
        self.doc_manager.remove_element(doc_id)
        # Process has failed status/does not exist, so document info is
        # deleted from the local database

    def _print_status(self, title, doc_id, progress, statustext):
        print ('{0} ({1}): {2}% ({3})'.format(title, doc_id, progress, statustext))
        # print title + ': ' + str(progress) + '%'
        # for each doc id, also call /document/id/translation and get % of each locale

    def _print_detailed_status(self, doc_id, doc_name):
        doc_id = self.get_latest_document_version(doc_id) or doc_id
        response = self.api.document_translation_status(doc_id)
        if response.status_code != 200:
            raise_error(response.json(), 'Failed to get detailed status of document', True, doc_id, doc_name)
        try:
            # print(response.json())
            if 'entities' in response.json():
                for entry in response.json()['entities']:
                    curr_locale = entry['properties']['locale_code']
                    curr_progress = entry['properties']['percent_complete']
                    curr_statustext = entry['properties']['status']
                    # print ('\tlocale: {0} \t percent complete: {1}%'.format(curr_locale, curr_progress))
                    if 'entities' in entry:
                        for entity in entry['entities']:
                            if entity['rel'][0] == 'phases':
                                if 'entities' in entity:
                                    table = []
                                    for phase in entity['entities']:
                                        phase_name = phase['properties']['name']
                                        phase_order = phase['properties']['order']
                                        phase_percent_complete = phase['properties']['percent_completed']
                                        phase_status = phase['properties']['status']
                                        table.append({"Phase": str(phase_order), "Name": phase_name, "Status": phase_status, "Phase Percent Complete": str(phase_percent_complete) + '%'})
                                    table.sort(key=lambda x: x['Phase'])
                                    print('\n')
                                    print('Locale: {0} \t Total Percent Complete: {1}% ({2})\n'.format(curr_locale, curr_progress, curr_statustext))
                                    # print('Locale: {0} \n'.format(curr_locale))
                                    print(tabulate(table, headers="keys"))
                                    
                    # detailed_status[doc_id] = (curr_locale, curr_progress)
        except KeyError as e:
            log_error(self.error_file_name, e)
            print("Error listing translations")
            return
            # return detailed_status



CONF_DIR = '.ltk'
CONF_FN = 'config'
TRANS_DIR = 'Lingotek Translations'
DB_FN = 'docs.json'
FOLDER_DB_FN = 'folders.json'
LOG_FN = 'log'
APP_ID = 'ab33b8b9-4c01-43bd-a209-b59f933e4fc4'
SYSTEM_FILE = '.lingotek'
ERROR_FN = 'error_log'
METADATA_FIELDS = ['author_email', 'author_name', 'business_division', 'business_unit', 'campaign_id', 'campaign_rating', 'channel', 'contact_email', 'contact_name', 'content_description', 'content_type', 'domain', 'external_application_id', 'external_document_id', 'external_style_id', 'job_id', 'purchase_order', 'reference_url', 'region', 'require_review', 'note']


import requests
from ltk.api_uri import API_URI
from ltk.utils import restart
import ltk.utils
from ltk.exceptions import RequestFailedError,ConnectionFailed
from ltk.logger import logger
import sys, os
# Python 2
# import urlparse as parse
# End Python 2
# Python 3
import urllib.parse as parse
# End Python 3

class ApiCalls:
    def __init__(self, host, access_token, watch=False, timeout=5):
        self.host = host
        self.headers = {'Authorization': 'bearer ' + access_token}
        self.watch = watch
        self.timeout = timeout
        self.cookie = ''
        # v To be removed v
        self.access_token = access_token
        # self.cert = ('lingotek.crt', 'lingotek.key')

    def handleError(self):
        if self.watch:
            logger.warning("Could not connect to Lingotek")
            restart("Restarting watch", self.timeout)
        else:
            raise ConnectionFailed("Could not connect to Lingotek")

# Returns true if successful in retrieving cookie, false if otherwise
    def access_login(self, host):
        try:
            uri = '/login'
            r = requests.get(host + uri)
            cookie = r.headers['set-cookie']
            log_api('GET', uri, r)
            if cookie and len(cookie) > 0:
                self.cookie = cookie
                return True
            else: return False
        except Exception as e:
            print("access_login", e)
            self.handleError()
            return False

# Returns true if access_login is successful, false if otherwise
    def login(self, host, username, password):
        output = self.access_login(host)
        if output == False: return False
        try:
            uri = '/login'
            payload = {'username': username, 'password': password}
            r = requests.post(host + uri, headers={'Cookie': self.cookie}, data=payload)
            log_api('POST', uri, r)
        except Exception as e:
            print("login", e)
            self.handleError()
        return output


# Returns access token if successful in retrieving it, None if otherwise
    def authenticate(self, host):
        output = None
        try:
            start = self.startup(host)
            if not start.url: return None
            query = parse.parse_qs(parse.urlparse(start.url).query)
            if 'client_id' in query.keys(): cid = query['client_id']
            else: return None
            uri = '/dialog/authorize'
            payload = {'redirect_uri':'https://myaccount.lingotek.com/tms-ui/html/portal/sso_redirect.html','response_type':'token','client_id':cid}
            # r = requests.get(host + uri, headers={'Host': 'cmssso.lingotek.com', 'Referer': 'https://cmssso.lingotek.com/login', 'Cache-Control':'max-age=0', 'Upgrade-Insecure-Requests':'1', 'Cookie':'__ctmid=58220c510010e8c8dc704410; _gat=1; _ga=GA1.2.831256021.1467748163; connect.sid=s%3AxU6QRRV9jDVSX3SeYAOElBOI1Y5HdMRK.yU%2FTgKno2PqlKGljl50dJ8HarhRUT71zT0rF6aniDvw'}, data=payload)
            # r = requests.get(host + uri, headers={'Cookie':'connect.sid=s%3Aq4dTUpbJVb8uIgbM7s2T0txtHR6qpkhE.5dFEBdjsPtlcDGgG9MO9yNQMhyrkMpJVjhLH84J2mKI'}, params=payload)
            r = requests.get(host + uri, headers={'Cookie': self.cookie}, params=payload)
            log_api('GET', uri, r)
            # r = requests.get(host + uri, headers=self.headers, params=payload)
            fragment = parse.parse_qs(parse.urlparse(r.url).fragment)
            if 'access_token' in fragment.keys() and len(fragment['access_token']) > 0: return fragment['access_token'][0]
            else: return None
        except Exception as e:
            print("authenticate", e)
            self.handleError()
            return None

# Returns a request object used in the authenticate function
    def startup(self, host):
        try:
            uri = '/lingopoint/portal/startup.action'
            r = requests.get(host + uri, headers={'Host':'myaccount.lingotek.com'})
            log_api('GET', uri, r)
        except Exception as e:
            print("startup", e)
            self.handleError()
        return r

    def list_communities(self):
        """ gets the communities that a user is in """
        try:
            uri = API_URI['community']
            payload = {'limit': 1000}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_document_formats(self):
        """ gets the communities that a user is in """
        try:
            uri = API_URI['document_format']
            payload = {'limit': 1000}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_projects(self, community_id):
        """ gets the projects a user has """
        try:
            uri = API_URI['project']
            payload = {'community_id': community_id, 'limit': 10000}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def add_project(self, project_name, community_id, workflow_id):
        """ adds a project """
        try:
            uri = API_URI['project']
            payload = {'title': project_name, 'community_id': community_id, 'workflow_id': workflow_id}
            r = requests.post(self.host + uri, headers=self.headers, data=payload)
            log_api('POST', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_project(self, project_id):
        """ Get a project which the active user has access to """
        try:
            uri = (API_URI['project_id'] % locals())
            payload = {}
            r = requests.get(self.host + uri, headers=self.headers, data=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()

        return r

    def patch_project(self, project_id, workflow_id):
        """ updates a project """
        try:
            uri = (API_URI['project_id'] % locals())
            payload = {'project_id': project_id}
            if workflow_id:
                payload['workflow_id'] = workflow_id
            r = requests.patch(self.host + uri, headers=self.headers, data=payload)
            log_api('PATCH', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def project_add_target(self, project_id, locale, due_date):
        """ adds a target to all documents within a project """
        try:
            uri = (API_URI['project_translation'] % locals())
            payload = {'id': project_id, 'locale_code': locale}
            if due_date:
                payload['due_date'] = due_date
            r = requests.post(self.host + uri, headers=self.headers, data=payload)
            log_api('POST', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def project_status(self, project_id):
        """ gets the status of a project """
        try:
            uri = (API_URI['project_status'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def project_delete_target(self, project_id, locale):
        try:
            uri = (API_URI['project_translation_locale'] % locals())
            r = requests.delete(self.host + uri, headers=self.headers)
            log_api('DELETE', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def delete_project(self, project_id):
        """ deletes a project """
        try:
            uri = (API_URI['project_id'] % locals())
            r = requests.delete(self.host + uri, headers=self.headers)
            log_api('DELETE', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_document(self, document_id):
        """ gets a document by id """
        try:
            uri = (API_URI['document_id'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_latest_document(self, document_id):
        """ get latest version of a document """
        try:
            uri = (API_URI['document_latest_version'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def add_document(self, source_locale, file_name, project_id, title, doc_metadata={}, **kwargs):
        """ adds a document """
        try:
            uri = API_URI['document']
            payload = {'locale_code': source_locale, 'project_id': project_id, 'title': title}
            for field in doc_metadata:
                if doc_metadata[field]:
                    payload[field] = doc_metadata[field]
            for key in kwargs:#set after metadata so metadata flags can override the wizard
                if kwargs[key]:
                    if key == 'download_folder' or key == 'overwrite':
                        continue #don't add this key because the API is not set up to do anything with it and we don't want to expose anything in an API post that we don't have to
                    payload[key] = kwargs[key]
            detected_format = ltk.utils.detect_format(file_name)
            if ('format' not in kwargs or kwargs['format'] is None):
                payload['format'] = detected_format
            document = open(file_name, 'rb')
            files = {'content': (file_name, document)}
            if 'srx' in kwargs and kwargs['srx'] is not None:
                files.update({'srx': (kwargs['srx'], open(kwargs['srx'], 'rb'))})
            if 'its' in kwargs and kwargs['its'] is not None:
                files.update({'its': (kwargs['its'], open(kwargs['its'], 'rb'))})
            if 'fprm' in kwargs and kwargs['fprm'] is not None:
                fprm_file = open(kwargs['fprm'], 'rb')
                files.update({'fprm': (kwargs['fprm'], fprm_file)})
            r = requests.post(self.host + uri, headers=self.headers, data=payload, files=files)
            log_api('POST', uri, r)
            document.close()
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r
    
    def document_add_reference(self, document_id, reference):
        """ adds reference material to a document """
        try:
            uri = (API_URI['reference'] % locals())
            payload = {'id': document_id}
            document = open(reference['file'], 'rb')
            files = {'content': (reference['file'], document)}
            if 'name' in reference and reference['name']:
                payload.update({'name': reference['name']})
            if 'description' in reference and reference['description']:
                payload.update({'description': reference['description']})
            r = requests.post(self.host + uri, headers=self.headers, data=payload, files=files)
            log_api('POST', uri, r)
            document.close()
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_list_reference(self, document_id):
        """ lists the reference material that is attached to a document """
        try:
            uri = (API_URI['reference'] % locals())
            payload = {'id': document_id}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_download_reference(self, document_id, reference_id):
        """ downloads reference material that is attached to a document """
        try:
            uri = (API_URI['reference_id'] % locals())
            payload = {'id': document_id, 'reference_id': reference_id}
            r = requests.get(self.host + uri, headers=self.headers, params=payload, stream=True)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r
    
    def document_remove_reference(self, document_id, reference_id):
       """ removes reference material from a document """
       try:
           uri = (API_URI['reference_id'] % locals())
           r = requests.delete(self.host + uri, headers=self.headers)
           log_api('DELETE', uri, r)
       except requests.exceptions.ConnectionError:
           self.handleError()
       return r

    def document_add_target(self, document_id, locale, workflow_id=None, due_date=None):
        """ adds a target to existing document, starts the workflow """
        try:
            uri = (API_URI['document_translation'] % locals())
            payload = {'locale_code': locale, 'id': document_id}
            if workflow_id:
                payload['workflow_id'] = workflow_id
            if due_date:
                payload['due_date'] = due_date
            r = requests.post(self.host + uri, headers=self.headers, data=payload)
            log_api('POST', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_documents(self, project_id):
        """ lists all documents a user has access to, could be filtered by project id """
        try:
            uri = API_URI['document']
            payload = {}
            if project_id:
                payload = {'project_id': project_id, 'limit': 10000}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_status(self, document_id):
        """ gets the status of a document """
        try:
            uri = (API_URI['document_status'] % locals())
            payload = {'document_id': document_id}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            # logger.debug(r.url)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_translation_status(self, document_id):
        """ gets the status of document translations """
        try:
            uri = (API_URI['document_translation'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_translation_locale_status(self, document_id, locale):
        """ gets the status of a specific target for a document translation """
        try:
            uri = (API_URI['document_translation_locale'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_content(self, document_id, locale_code, auto_format, xliff=False, finalized_file='off'):
        """ downloads the translated document """
        headers = self.headers
        try:
            uri = (API_URI['document_content'] % locals())
            payload = {}
            payload['auto_format'] = auto_format
            if locale_code:
                payload['locale_code'] = locale_code
            if xliff:
                headers['Accept'] = 'application/x-xliff+xml'
            if finalized_file == 'on':
                payload['finalized_file'] = 'true'

            r = requests.get(self.host + uri, headers=headers, params=payload, stream=True)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_update(self, document_id, file_name=None, doc_metadata={}, **kwargs):
        try:
            uri = (API_URI['document_id'] % locals())
            payload = {'id': document_id}
            for field in doc_metadata:
                if doc_metadata[field]:
                    payload[field] = doc_metadata[field]
            for key in kwargs:#set after metadata so metadata flags can override the metadata wizard
                if kwargs[key]:
                    payload[key] = kwargs[key]#will need to update this to look like it does in add_document if any arguments are added to ltk push that take parameters but shouldn't be sent
            if file_name:
                document = open(file_name, 'rb')
                files = {'content': (file_name, document)}
                if 'srx' in kwargs and kwargs['srx'] is not None:
                    files.update({'srx': (kwargs['srx'], open(kwargs['srx'], 'rb'))})
                if 'its' in kwargs and kwargs['its'] is not None:
                    files.update({'its': (kwargs['its'], open(kwargs['its'], 'rb'))})
                if 'fprm' in kwargs and kwargs['fprm'] is not None:
                    files.update({'fprm': (kwargs['fprm'], open(kwargs['fprm'], 'rb'))})
                r = requests.patch(self.host + uri, headers=self.headers, data=payload, files=files)
                document.close()
            else:
                r = requests.patch(self.host + uri, headers=self.headers, data=payload)
            log_api('PATCH', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_delete_target(self, document_id, locale):
        try:
            uri = (API_URI['document_translation_locale'] % locals())
            r = requests.delete(self.host + uri, headers=self.headers)
            log_api('DELETE', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_cancel_target(self, document_id, locale):
        try:
            uri = (API_URI['document_cancel_locale'] % locals())
            payload = {'id': document_id, 'locale': locale, 'cancelled_reason': 'CANCELLED_BY_AUTHOR', 'mark_invoiceable': 'true'}
            r = requests.post(self.host + uri, headers=self.headers, data=payload)
            log_api('POST', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_delete(self, document_id):
        try:
            uri = (API_URI['document_id'] % locals())
            r = requests.delete(self.host + uri, headers=self.headers)
            log_api('DELETE', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def document_cancel(self, document_id):
        try:
            uri = (API_URI['document_cancel'] % locals())
            payload = {'id': document_id, 'cancelled_reason': 'CANCELLED_BY_AUTHOR'}
            r = requests.post(self.host + uri, headers=self.headers, data=payload)
            log_api('POST', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_process(self, process_id):
        try:
            uri = (API_URI['process'] % locals())
            payload = {'process_id': process_id}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_workflows(self, community_id):
        try:
            uri = API_URI['workflow']
            payload = {'community_id': community_id, 'limit': 1000}
            r = requests.get(self.host + uri, headers=self.headers, params=payload)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_locales(self):
        try:
            uri = 'http://gmc.lingotek.com/v1/locales'
            r = requests.get(uri)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def list_filters(self):
        try:
            uri = API_URI['filter']
            r = requests.get(self.host + uri, headers=self.headers, params={'limit': 1000})
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_filter_content(self, filter_id):
        """ gets a filter by id """
        try:
            uri = (API_URI['filter_content'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            if r.status_code == 200:
                log_api('GET', uri, r)
            else:
                log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_filter_info(self, filter_id):
        """ gets a filter by id """
        try:
            uri = (API_URI['filter_id'] % locals())
            r = requests.get(self.host + uri, headers=self.headers)
            log_api('GET', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def patch_filter(self, filter_id, filename):
        """ update a filter by id """
        try:
            uri = (API_URI['filter_id'] % locals())
            params = {'id': filter_id}
            content = open(filename, 'rb')
            files = {'content': (filename, content)}
            r = requests.patch(self.host + uri, headers=self.headers, data=params, files=files)
            log_api('PATCH', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def post_filter(self, filename, filter_type='FPRM'):
        """ post a filter """
        try:
            uri = API_URI['filter']
            content = open(filename, 'rb')
            files = {'content': (filename, content)}
            if filter_type is None:
                extension = os.path.splitext(filename)[1]
                filter_type = extension[1:]
            filter_type = filter_type.lower()
            name = os.path.splitext(filename)[0] #remove extension from filename
            params = {'name': name, 'type': filter_type}
            r = requests.post(self.host + uri, headers=self.headers, data=params, files=files)
            log_api('PATCH', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def delete_filter(self, filter_id):
        """ deletes a filter """
        try:
            uri = (API_URI['filter_id'] % locals())
            r = requests.delete(self.host + uri, headers=self.headers)
            log_api('DELETE', uri, r)
        except requests.exceptions.ConnectionError:
            self.handleError()
        return r

    def get_project_info(self, community_id):
        response = self.list_projects(community_id)
        info = {}
        if response.status_code == 200:
            if int(response.json()['properties']['total']) == 0:
                return info
            entities = response.json()['entities']
            for entity in entities:
                info[entity['properties']['id']] = entity['properties']['title']
            return info
        elif response.status_code == 204:
            return info
        else:
            raise RequestFailedError("Unable to get existing projects")

    def get_communities_info(self):
        response = self.list_communities()
        if response.status_code != 200:
            # raise RequestFailedError("Unable to get user's list of communities")
            return None # Don't stop execution upon incorrect access token
        entities = response.json()['entities']
        info = {}
        for entity in entities:
            info[entity['properties']['id']] = entity['properties']['title']
        return info

    def get_document_formats(self):
        response = self.list_document_formats()
        info = {}
        if response.status_code != 200:
            # raise RequestFailedError("Unable to get document formats")
            return info # Don't stop execution upon incorrect access token
        entities = response.json()['entities']
        for entity in entities:
            info[entity['properties']['type']] = entity['properties']['type']
        return info


def log_api(method, uri, response):
    try:
        content_length = response.headers['content-length']
    except KeyError:
        content_length = 'No content-length header'
    log = '{method} {uri} {status} {length}'.format(method=method, uri=uri, status=response.status_code,
                                                    length=content_length)
    logger.api_call(log)

    if 'content-type' in response.headers and response.headers['content-type'].find("json") != -1:
        try:
            import json
            response_info = json.dumps(response.json(), indent=4, separators=(',', ': '))
        except ValueError:
            response_info = 'No json response available'
        logger.api_response(response_info)
    elif 'content-length' in response.headers and response.headers['content-length'] != '0':
        if response.headers['content-type'] == 'application/zip':
            logger.api_response('Zip file')
        else:
            logger.api_response(response.content.decode('UTF-8'))
